@@ -19,18 +19,16 @@
 
 /**
  * SECTION:element-curlsink
- * @title: curlsink
  * @short_description: sink that uploads data to a server using libcurl
+ * @see_also:
  *
  * This is a network sink that uses libcurl as a client to upload data to
  * an SMTP server.
  *
- * ## Example launch line
- *
- * Upload a JPEG file to an SMTP server.
- *
+ * <refsect2>
+ * <title>Example launch line (upload a JPEG file to an SMTP server)</title>
  * |[
- * gst-launch-1.0 filesrc location=image.jpg ! jpegparse ! curlsmtpsink  \
+ * gst-launch filesrc location=image.jpg ! jpegparse ! curlsmtpsink  \
  *     file-name=image.jpg  \
  *     location=smtp://smtp.gmail.com:507 \
  *     user=test passwd=test  \
@@ -40,7 +38,7 @@
  *     use-ssl=TRUE  \
  *     insecure=TRUE
  * ]|
- *
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,29 +49,27 @@
 #include <string.h>
 #include <stdio.h>
 
-#ifdef HAVE_SYS_SOCKET_H
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
 #include <sys/types.h>
-#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-#endif
-#ifdef HAVE_PWD_H
+#if HAVE_PWD_H
 #include <pwd.h>
 #endif
-#ifdef HAVE_NETINET_IN_H
+#if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_NETINET_IP_H
+#include <unistd.h>
+#if HAVE_NETINET_IP_H
 #include <netinet/ip.h>
 #endif
-#ifdef HAVE_NETINET_TCP_H
+#if HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
 #endif
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include "gstcurlelements.h"
 #include "gstcurltlssink.h"
 #include "gstcurlsmtpsink.h"
 
@@ -136,8 +132,6 @@ static size_t transfer_payload_headers (GstCurlSmtpSink * sink, void *curl_ptr,
 
 #define gst_curl_smtp_sink_parent_class parent_class
 G_DEFINE_TYPE (GstCurlSmtpSink, gst_curl_smtp_sink, GST_TYPE_CURL_TLS_SINK);
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (curlsmtpsink, "curlsmtpsink",
-    GST_RANK_NONE, GST_TYPE_CURL_SMTP_SINK, curl_element_init (plugin));
 
 static void
 gst_curl_smtp_sink_notify_transfer_end_unlocked (GstCurlSmtpSink * sink)
@@ -208,16 +202,21 @@ gst_curl_smtp_sink_event (GstBaseSink * bsink, GstEvent * event)
 
       GST_OBJECT_LOCK (sink);
       sink->eos = TRUE;
-      if (bcsink->flow_ret == GST_FLOW_OK && sink->base64_chunk != NULL
-          && !sink->final_boundary_added) {
+      GST_OBJECT_UNLOCK (sink);
+
+      if (sink->base64_chunk != NULL)
         add_final_boundary_unlocked (sink);
-        gst_curl_base_sink_transfer_thread_notify_unlocked (bcsink);
-        GST_FIXME_OBJECT (sink, "if gstpoll errors in transfer thread, then "
-            "this wait will never timeout because the transfer thread does "
-            "not signal it upon errors");
+
+      gst_curl_base_sink_transfer_thread_notify_unlocked (bcsink);
+
+      GST_OBJECT_LOCK (sink);
+      if (sink->base64_chunk != NULL && bcsink->flow_ret == GST_FLOW_OK) {
         gst_curl_smtp_sink_wait_for_transfer_end_unlocked (sink);
       }
       GST_OBJECT_UNLOCK (sink);
+
+      gst_curl_base_sink_transfer_thread_close (bcsink);
+
       break;
 
     default:
@@ -256,6 +255,7 @@ gst_curl_smtp_sink_class_init (GstCurlSmtpSinkClass * klass)
 
   GST_DEBUG_CATEGORY_INIT (gst_curl_smtp_sink_debug, "curlsmtpsink", 0,
       "curl smtp sink element");
+  GST_DEBUG_OBJECT (klass, "class_init");
 
   gst_element_class_set_static_metadata (element_class,
       "Curl smtp sink",
@@ -433,7 +433,7 @@ gst_curl_smtp_sink_set_property (GObject * object, guint prop_id,
         break;
       case PROP_NBR_ATTACHMENTS:
         sink->nbr_attachments = g_value_get_int (value);
-        sink->curr_attachment = 1;
+        sink->nbr_attachments_left = sink->nbr_attachments;
         GST_DEBUG_OBJECT (sink, "nbr-attachments set to %d",
             sink->nbr_attachments);
         break;
@@ -638,7 +638,7 @@ gst_curl_smtp_sink_set_transfer_options_unlocked (GstCurlBaseSink * bcsink)
 
   /* time */
   date = g_date_time_new_now_local ();
-  date_str = g_date_time_format (date, "%a, %_e %b %Y %H:%M:%S %z");
+  date_str = g_date_time_format (date, "%a %b %e %H:%M:%S %Y %z");
   g_date_time_unref (date);
 
   /* recipient, sender and subject are all UTF-8 strings, which are additionally
@@ -725,13 +725,6 @@ gst_curl_smtp_sink_set_transfer_options_unlocked (GstCurlBaseSink * bcsink)
     return FALSE;
   }
 
-  res = curl_easy_setopt (bcsink->curl, CURLOPT_UPLOAD, 1L);
-  if (res != CURLE_OK) {
-    bcsink->error = g_strdup_printf ("failed to prepare for upload: %s",
-        curl_easy_strerror (res));
-    return FALSE;
-  }
-
   parent_class = GST_CURL_TLS_SINK_GET_CLASS (sink);
 
   if (sink->use_ssl) {
@@ -773,19 +766,18 @@ gst_curl_smtp_sink_flush_data_unlocked (GstCurlBaseSink * bcsink,
   gchar *data_out;
 
   GST_DEBUG
-      ("live: %d, num attachments: %d, curr_attachment: %d, "
-      "eos: %d, close_transfer: %d, final boundary: %d, array_len: %d",
-      bcsink->is_live, sink->nbr_attachments, sink->curr_attachment,
-      sink->eos, close_transfer, sink->final_boundary_added, array->len);
+      ("live: %d, num attachments: %d, num attachments_left: %d, eos: %d, "
+      "close_transfer: %d, final boundary: %d, array_len: %d", bcsink->is_live,
+      sink->nbr_attachments, sink->nbr_attachments_left, sink->eos,
+      close_transfer, sink->final_boundary_added, array->len);
 
 
-  if ((bcsink->is_live && (sink->curr_attachment == sink->nbr_attachments))
+  if ((bcsink->is_live && (sink->nbr_attachments_left == sink->nbr_attachments))
       || (sink->nbr_attachments == 1) || sink->eos
       || sink->final_boundary_added) {
     bcsink->is_live = FALSE;
     sink->reset_transfer_options = TRUE;
     sink->final_boundary_added = FALSE;
-    sink->curr_attachment = 1;
 
     GST_DEBUG ("returning 0, no more data to send in this transfer");
 
@@ -807,8 +799,12 @@ gst_curl_smtp_sink_flush_data_unlocked (GstCurlBaseSink * bcsink,
   g_free (data_out);
 
   if (new_file) {
-    sink->curr_attachment++;
+    sink->nbr_attachments_left--;
+
     bcsink->is_live = TRUE;
+    if (sink->nbr_attachments_left <= 1) {
+      sink->nbr_attachments_left = sink->nbr_attachments;
+    }
 
     /* reset flag */
     bcsink->new_file = FALSE;
@@ -926,16 +922,6 @@ gst_curl_smtp_sink_transfer_data_buffer (GstCurlBaseSink * bcsink,
         transfer_chunk (curl_ptr, bcsink->transfer_buf, sink->base64_chunk,
         block_size, last_chunk);
 
-    /* if last chunk of current buffer and max attachments per mail is reached
-     * then add final boundary */
-    if (*last_chunk && sink->curr_attachment == sink->nbr_attachments &&
-        !sink->final_boundary_added) {
-      add_final_boundary_unlocked (sink);
-      /* now that we've added the final boundary to the array we have on more
-       * chunk to send */
-      *last_chunk = 0;
-    }
-
     GST_OBJECT_LOCK (sink);
     if (sink->eos) {
       gst_curl_smtp_sink_notify_transfer_end_unlocked (sink);
