@@ -163,10 +163,10 @@ facedetect_update_get_type (void)
     {GST_FACEDETECT_UPDATES_EVERY_FRAME, "Send update messages on every frame",
         "every_frame"},
     {GST_FACEDETECT_UPDATES_ON_CHANGE,
-          "Send messages when a new face is detected or one is not anymore detected",
+          "Send update messages on change (face detected/not detected)",
         "on_change"},
     {GST_FACEDETECT_UPDATES_ON_FACE,
-          "Send messages whenever a face is detected",
+          "Send update messages when a face is detected",
         "on_face"},
     {GST_FACEDETECT_UPDATES_NONE, "Send no messages update", "none"},
     {0, NULL, NULL},
@@ -561,8 +561,7 @@ gst_face_detect_transform_ip (GstOpencvVideoFilter * base, GstBuffer * buf,
         break;
       case GST_FACEDETECT_UPDATES_ON_CHANGE:
         if (faces && faces->total > 0) {
-          if (!filter->face_detected)
-            post_msg = TRUE;
+          post_msg = TRUE;
         } else {
           if (filter->face_detected) {
             post_msg = TRUE;
