@@ -18,17 +18,22 @@
 /**
  * SECTION:element-openni2src
  *
- * ## Examples
+ * <refsect2>
+ * <title>Examples</title>
+ * <para>
+ * Some recorded .oni files are available at:
+ * <programlisting>
+ *  http://people.cs.pitt.edu/~chang/1635/proj11/kinectRecord
+ * </programlisting>
  *
- * Some recorded .oni files are available at <http://people.cs.pitt.edu/~chang/1635/proj11/kinectRecord>
- *
- * ``` shell
- * LD_LIBRARY_PATH=/usr/lib/OpenNI2/Drivers/ gst-launch-1.0 --gst-debug=openni2src:5   openni2src location='Downloads/mr.oni' sourcetype=depth ! videoconvert ! ximagesink
- * ```
- *
- * ``` shell
- * LD_LIBRARY_PATH=/usr/lib/OpenNI2/Drivers/ gst-launch-1.0 --gst-debug=openni2src:5   openni2src location='Downloads/mr.oni' sourcetype=color ! videoconvert ! ximagesink
- * ```
+ * <programlisting>
+  LD_LIBRARY_PATH=/usr/lib/OpenNI2/Drivers/ gst-launch-1.0 --gst-debug=openni2src:5   openni2src location='Downloads/mr.oni' sourcetype=depth ! videoconvert ! ximagesink
+ * </programlisting>
+ * <programlisting>
+  LD_LIBRARY_PATH=/usr/lib/OpenNI2/Drivers/ gst-launch-1.0 --gst-debug=openni2src:5   openni2src location='Downloads/mr.oni' sourcetype=color ! videoconvert ! ximagesink
+ * </programlisting>
+ * </para>
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -111,8 +116,6 @@ static GstFlowReturn openni2_read_gstbuffer (GstOpenni2Src * src,
 
 #define parent_class gst_openni2_src_parent_class
 G_DEFINE_TYPE (GstOpenni2Src, gst_openni2_src, GST_TYPE_PUSH_SRC);
-GST_ELEMENT_REGISTER_DEFINE (openni2src, "openni2src", GST_RANK_NONE,
-    GST_TYPE_OPENNI2_SRC);
 
 static void
 gst_openni2_src_class_init (GstOpenni2SrcClass * klass)
@@ -142,8 +145,6 @@ gst_openni2_src_class_init (GstOpenni2SrcClass * klass)
           GST_TYPE_OPENNI2_SRC_SOURCETYPE, DEFAULT_SOURCETYPE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
-  gst_type_mark_as_plugin_api (GST_TYPE_OPENNI2_SRC_SOURCETYPE,
-      (GstPluginAPIFlags) 0);
 
   basesrc_class->start = GST_DEBUG_FUNCPTR (gst_openni2_src_start);
   basesrc_class->stop = GST_DEBUG_FUNCPTR (gst_openni2_src_stop);
@@ -152,7 +153,8 @@ gst_openni2_src_class_init (GstOpenni2SrcClass * klass)
   basesrc_class->decide_allocation =
       GST_DEBUG_FUNCPTR (gst_openni2src_decide_allocation);
 
-  gst_element_class_add_static_pad_template (element_class, &srctemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&srctemplate));
 
   gst_element_class_set_static_metadata (element_class, "Openni2 client source",
       "Source/Video",
@@ -306,7 +308,7 @@ gst_openni2_src_start (GstBaseSrc * bsrc)
   if (src->depth->isValid ()) {
     rc = src->depth->start ();
     if (rc != openni::STATUS_OK) {
-      GST_ERROR_OBJECT (src, "Couldn't start the depth stream: %s",
+      GST_ERROR_OBJECT (src, "Couldn't start the depth stream\n%s\n",
           openni::OpenNI::getExtendedError ());
       return FALSE;
     }
@@ -315,7 +317,7 @@ gst_openni2_src_start (GstBaseSrc * bsrc)
   if (src->color->isValid ()) {
     rc = src->color->start ();
     if (rc != openni::STATUS_OK) {
-      GST_ERROR_OBJECT (src, "Couldn't start the color stream: %s",
+      GST_ERROR_OBJECT (src, "Couldn't start the color stream\n%s\n",
           openni::OpenNI::getExtendedError ());
       return FALSE;
     }
@@ -509,11 +511,17 @@ gst_openni2src_decide_allocation (GstBaseSrc * bsrc, GstQuery * query)
   else
     gst_query_add_allocation_pool (query, pool, size, min, max);
 
-  gst_object_unref (pool);
-
   return GST_BASE_SRC_CLASS (parent_class)->decide_allocation (bsrc, query);
 }
 
+gboolean
+gst_openni2src_plugin_init (GstPlugin * plugin)
+{
+  return gst_element_register (plugin, "openni2src", GST_RANK_NONE,
+      GST_TYPE_OPENNI2_SRC);
+}
+
+
 static gboolean
 openni2_initialise_library (void)
 {
@@ -545,7 +553,7 @@ openni2_initialise_devices (GstOpenni2Src * src)
     return FALSE;
   }
 
-  /* depth sensor */
+  /** depth sensor **/
   rc = src->depth->create (*src->device, openni::SENSOR_DEPTH);
   if (rc == openni::STATUS_OK) {
     rc = src->depth->start ();
@@ -558,7 +566,7 @@ openni2_initialise_devices (GstOpenni2Src * src)
         openni::OpenNI::getExtendedError ());
   }
 
-  /* color sensor */
+  /** color sensor **/
   rc = src->color->create (*src->device, openni::SENSOR_COLOR);
   if (rc == openni::STATUS_OK) {
     rc = src->color->start ();
@@ -573,12 +581,12 @@ openni2_initialise_devices (GstOpenni2Src * src)
   }
 
   if (!src->depth->isValid () && !src->color->isValid ()) {
-    GST_ERROR_OBJECT (src, "No valid streams. Exiting");
+    GST_ERROR_OBJECT (src, "No valid streams. Exiting\n");
     openni::OpenNI::shutdown ();
     return FALSE;
   }
 
-  /* Get resolution and make sure is valid */
+  /** Get resolution and make sure is valid **/
   if (src->depth->isValid () && src->color->isValid ()) {
     src->depthVideoMode = src->depth->getVideoMode ();
     src->colorVideoMode = src->color->getVideoMode ();
@@ -736,9 +744,6 @@ openni2_read_gstbuffer (GstOpenni2Src * src, GstBuffer * buf)
         src->colorFrame->getWidth (),
         src->colorFrame->getHeight (),
         src->colorFrame->getDataSize ());
-  } else {
-    g_return_val_if_reached (GST_FLOW_ERROR);
-    return GST_FLOW_ERROR;
   }
 
   if (G_UNLIKELY (src->oni_start_ts == GST_CLOCK_TIME_NONE))
