@@ -32,7 +32,7 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! opusenc ! oggmux ! filesink location=sine.ogg
+ * gst-launch -v audiotestsrc wave=sine num-buffers=100 ! audioconvert ! opusenc ! oggmux ! filesink location=sine.ogg
  * ]| Encode a test sine signal to Ogg/OPUS.
  * </refsect2>
  */
@@ -44,12 +44,10 @@
 #include <string.h>
 #include <time.h>
 #include <math.h>
-#include <opus.h>
+#include <opus/opus.h>
 
 #include <gst/gsttagsetter.h>
 #include <gst/audio/audio.h>
-#include <gst/pbutils/pbutils.h>
-#include <gst/tag/tag.h>
 #include <gst/glib-compat-private.h>
 #include "gstopusheader.h"
 #include "gstopuscommon.h"
@@ -116,51 +114,6 @@ gst_opus_enc_frame_size_get_type (void)
   return id;
 }
 
-#define GST_OPUS_ENC_TYPE_AUDIO_TYPE (gst_opus_enc_audio_type_get_type())
-static GType
-gst_opus_enc_audio_type_get_type (void)
-{
-  static const GEnumValue values[] = {
-    {OPUS_APPLICATION_AUDIO, "Generic audio", "generic"},
-    {OPUS_APPLICATION_VOIP, "Voice", "voice"},
-    {0, NULL, NULL}
-  };
-  static volatile GType id = 0;
-
-  if (g_once_init_enter ((gsize *) & id)) {
-    GType _id;
-
-    _id = g_enum_register_static ("GstOpusEncAudioType", values);
-
-    g_once_init_leave ((gsize *) & id, _id);
-  }
-
-  return id;
-}
-
-#define GST_OPUS_ENC_TYPE_BITRATE_TYPE (gst_opus_enc_bitrate_type_get_type())
-static GType
-gst_opus_enc_bitrate_type_get_type (void)
-{
-  static const GEnumValue values[] = {
-    {BITRATE_TYPE_CBR, "CBR", "cbr"},
-    {BITRATE_TYPE_VBR, "VBR", "vbr"},
-    {BITRATE_TYPE_CONSTRAINED_VBR, "Constrained VBR", "constrained-vbr"},
-    {0, NULL, NULL}
-  };
-  static volatile GType id = 0;
-
-  if (g_once_init_enter ((gsize *) & id)) {
-    GType _id;
-
-    _id = g_enum_register_static ("GstOpusEncBitrateType", values);
-
-    g_once_init_leave ((gsize *) & id, _id);
-  }
-
-  return id;
-}
-
 #define FORMAT_STR GST_AUDIO_NE(S16)
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -168,13 +121,8 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("audio/x-raw, "
         "format = (string) " FORMAT_STR ", "
         "layout = (string) interleaved, "
-        "rate = (int) 48000, "
-        "channels = (int) [ 1, 8 ]; "
-        "audio/x-raw, "
-        "format = (string) " FORMAT_STR ", "
-        "layout = (string) interleaved, "
-        "rate = (int) { 8000, 12000, 16000, 24000 }, "
-        "channels = (int) [ 1, 8 ] ")
+        "rate = (int) { 8000, 12000, 16000, 24000, 48000 }, "
+        "channels = (int) [ 1, 2 ] ")
     );
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
@@ -184,13 +132,11 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     );
 
 #define DEFAULT_AUDIO           TRUE
-#define DEFAULT_AUDIO_TYPE      OPUS_APPLICATION_AUDIO
 #define DEFAULT_BITRATE         64000
 #define DEFAULT_BANDWIDTH       OPUS_BANDWIDTH_FULLBAND
 #define DEFAULT_FRAMESIZE       20
 #define DEFAULT_CBR             TRUE
 #define DEFAULT_CONSTRAINED_VBR TRUE
-#define DEFAULT_BITRATE_TYPE    BITRATE_TYPE_CBR
 #define DEFAULT_COMPLEXITY      10
 #define DEFAULT_INBAND_FEC      FALSE
 #define DEFAULT_DTX             FALSE
@@ -201,13 +147,11 @@ enum
 {
   PROP_0,
   PROP_AUDIO,
-  PROP_AUDIO_TYPE,
   PROP_BITRATE,
   PROP_BANDWIDTH,
   PROP_FRAME_SIZE,
   PROP_CBR,
   PROP_CONSTRAINED_VBR,
-  PROP_BITRATE_TYPE,
   PROP_COMPLEXITY,
   PROP_INBAND_FEC,
   PROP_DTX,
@@ -228,7 +172,6 @@ static void gst_opus_enc_get_property (GObject * object, guint prop_id,
 static void gst_opus_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 
-static void gst_opus_enc_set_tags (GstOpusEnc * enc);
 static gboolean gst_opus_enc_start (GstAudioEncoder * benc);
 static gboolean gst_opus_enc_stop (GstAudioEncoder * benc);
 static gboolean gst_opus_enc_set_format (GstAudioEncoder * benc,
@@ -244,22 +187,6 @@ G_DEFINE_TYPE_WITH_CODE (GstOpusEnc, gst_opus_enc, GST_TYPE_AUDIO_ENCODER,
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL);
     G_IMPLEMENT_INTERFACE (GST_TYPE_PRESET, NULL));
 
-static void
-gst_opus_enc_set_tags (GstOpusEnc * enc)
-{
-  GstTagList *taglist;
-
-  /* create a taglist and add a bitrate tag to it */
-  taglist = gst_tag_list_new_empty ();
-  gst_tag_list_add (taglist, GST_TAG_MERGE_REPLACE,
-      GST_TAG_BITRATE, enc->bitrate, NULL);
-
-  gst_audio_encoder_merge_tags (GST_AUDIO_ENCODER (enc), taglist,
-      GST_TAG_MERGE_REPLACE);
-
-  gst_tag_list_unref (taglist);
-}
-
 static void
 gst_opus_enc_class_init (GstOpusEncClass * klass)
 {
@@ -291,18 +218,13 @@ gst_opus_enc_class_init (GstOpusEncClass * klass)
   base_class->getcaps = GST_DEBUG_FUNCPTR (gst_opus_enc_sink_getcaps);
 
   g_object_class_install_property (gobject_class, PROP_AUDIO,
-      g_param_spec_boolean ("audio",
-          "Audio or voice",
-          "Audio or voice (DEPRECATED: use audio-type)", DEFAULT_AUDIO,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_DEPRECATED));
-  g_object_class_install_property (gobject_class, PROP_AUDIO_TYPE,
-      g_param_spec_enum ("audio-type", "What type of audio to optimize for",
-          "What type of audio to optimize for", GST_OPUS_ENC_TYPE_AUDIO_TYPE,
-          DEFAULT_AUDIO_TYPE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+      g_param_spec_boolean ("audio", "Audio or voice",
+          "Audio or voice", DEFAULT_AUDIO,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BITRATE,
       g_param_spec_int ("bitrate", "Encoding Bit-rate",
-          "Specify an encoding bit-rate (in bps).", LOWEST_BITRATE,
-          HIGHEST_BITRATE, DEFAULT_BITRATE,
+          "Specify an encoding bit-rate (in bps).",
+          LOWEST_BITRATE, HIGHEST_BITRATE, DEFAULT_BITRATE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
           GST_PARAM_MUTABLE_PLAYING));
   g_object_class_install_property (gobject_class, PROP_BANDWIDTH,
@@ -317,19 +239,13 @@ gst_opus_enc_class_init (GstOpusEncClass * klass)
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
           GST_PARAM_MUTABLE_PLAYING));
   g_object_class_install_property (gobject_class, PROP_CBR,
-      g_param_spec_boolean ("cbr", "Constant bit rate",
-          "Constant bit rate (DEPRECATED: use bitrate-type)", DEFAULT_CBR,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_MUTABLE_PLAYING
-          | G_PARAM_DEPRECATED));
+      g_param_spec_boolean ("cbr", "Constant bit rate", "Constant bit rate",
+          DEFAULT_CBR,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
+          GST_PARAM_MUTABLE_PLAYING));
   g_object_class_install_property (gobject_class, PROP_CONSTRAINED_VBR,
       g_param_spec_boolean ("constrained-vbr", "Constrained VBR",
-          "Constrained VBR (DEPRECATED: use bitrate-type)",
-          DEFAULT_CONSTRAINED_VBR,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | GST_PARAM_MUTABLE_PLAYING
-          | G_PARAM_DEPRECATED));
-  g_object_class_install_property (gobject_class, PROP_BITRATE_TYPE,
-      g_param_spec_enum ("bitrate-type", "Bitrate type", "Bitrate type",
-          GST_OPUS_ENC_TYPE_BITRATE_TYPE, DEFAULT_BITRATE_TYPE,
+          "Constrained VBR", DEFAULT_CONSTRAINED_VBR,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
           GST_PARAM_MUTABLE_PLAYING));
   g_object_class_install_property (gobject_class, PROP_COMPLEXITY,
@@ -379,9 +295,9 @@ gst_opus_enc_finalize (GObject * object)
 static void
 gst_opus_enc_init (GstOpusEnc * enc)
 {
-  GST_DEBUG_OBJECT (enc, "init");
+  GstAudioEncoder *benc = GST_AUDIO_ENCODER (enc);
 
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (enc));
+  GST_DEBUG_OBJECT (enc, "init");
 
   g_mutex_init (&enc->property_lock);
 
@@ -392,13 +308,17 @@ gst_opus_enc_init (GstOpusEnc * enc)
   enc->bitrate = DEFAULT_BITRATE;
   enc->bandwidth = DEFAULT_BANDWIDTH;
   enc->frame_size = DEFAULT_FRAMESIZE;
-  enc->bitrate_type = DEFAULT_BITRATE_TYPE;
+  enc->cbr = DEFAULT_CBR;
+  enc->constrained_vbr = DEFAULT_CONSTRAINED_VBR;
   enc->complexity = DEFAULT_COMPLEXITY;
   enc->inband_fec = DEFAULT_INBAND_FEC;
   enc->dtx = DEFAULT_DTX;
   enc->packet_loss_percentage = DEFAULT_PACKET_LOSS_PERCENT;
   enc->max_payload_size = DEFAULT_MAX_PAYLOAD_SIZE;
-  enc->audio_type = DEFAULT_AUDIO_TYPE;
+
+  /* arrange granulepos marking (and required perfect ts) */
+  gst_audio_encoder_set_mark_granule (benc, TRUE);
+  gst_audio_encoder_set_perfect_timestamp (benc, TRUE);
 }
 
 static gboolean
@@ -407,8 +327,8 @@ gst_opus_enc_start (GstAudioEncoder * benc)
   GstOpusEnc *enc = GST_OPUS_ENC (benc);
 
   GST_DEBUG_OBJECT (enc, "start");
-  enc->encoded_samples = 0;
-  enc->consumed_samples = 0;
+  enc->tags = gst_tag_list_new_empty ();
+  enc->header_sent = FALSE;
 
   return TRUE;
 }
@@ -419,10 +339,16 @@ gst_opus_enc_stop (GstAudioEncoder * benc)
   GstOpusEnc *enc = GST_OPUS_ENC (benc);
 
   GST_DEBUG_OBJECT (enc, "stop");
+  enc->header_sent = FALSE;
   if (enc->state) {
     opus_multistream_encoder_destroy (enc->state);
     enc->state = NULL;
   }
+  gst_tag_list_unref (enc->tags);
+  enc->tags = NULL;
+  g_slist_foreach (enc->headers, (GFunc) gst_buffer_unref, NULL);
+  g_slist_free (enc->headers);
+  enc->headers = NULL;
   gst_tag_setter_reset_tags (GST_TAG_SETTER (enc));
 
   return TRUE;
@@ -547,20 +473,27 @@ gst_opus_enc_setup_channel_mappings (GstOpusEnc * enc,
   /* For two channels, use the basic RTP mapping if the channels are
      mapped as left/right. */
   if (enc->n_channels == 2) {
-    GST_INFO_OBJECT (enc, "Stereo, trivial RTP mapping");
-    enc->channel_mapping_family = 0;
-    enc->n_stereo_streams = 1;
-    /* implicit mapping for family 0 */
-    return;
+    if (MAPS (0, FRONT_LEFT) && MAPS (1, FRONT_RIGHT)) {
+      GST_INFO_OBJECT (enc, "Stereo, canonical mapping");
+      enc->channel_mapping_family = 0;
+      enc->n_stereo_streams = 1;
+      /* The channel mapping is implicit for family 0, that's why we do not
+         attempt to create one for right/left - this will be mapped to the
+         Vorbis mapping below. */
+      return;
+    } else {
+      GST_DEBUG_OBJECT (enc, "Stereo, but not canonical mapping, continuing");
+    }
   }
 
-  /* For channels between 3 and 8, we use the Vorbis mapping if we can
-     find a permutation that matches it. Mono and stereo will have been taken
-     care of earlier, but this code also handles it. There are two mappings.
-     One maps the input channels to an ordering which has the natural pairs
-     first so they can benefit from the Opus stereo channel coupling, and the
-     other maps this ordering to the Vorbis ordering. */
-  if (enc->n_channels >= 3 && enc->n_channels <= 8) {
+  /* For channels between 1 and 8, we use the Vorbis mapping if we can
+     find a permutation that matches it. Mono will have been taken care
+     of earlier, but this code also handles it. Same for left/right stereo.
+     There are two mappings. One maps the input channels to an ordering
+     which has the natural pairs first so they can benefit from the Opus
+     stereo channel coupling, and the other maps this ordering to the
+     Vorbis ordering. */
+  if (enc->n_channels >= 1 && enc->n_channels <= 8) {
     int c0, c1, c0v, c1v;
     int mapped;
     gboolean positions_done[256];
@@ -575,8 +508,6 @@ gst_opus_enc_setup_channel_mappings (GstOpusEnc * enc,
           GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER},
       {GST_AUDIO_CHANNEL_POSITION_SIDE_LEFT,
           GST_AUDIO_CHANNEL_POSITION_SIDE_RIGHT},
-      {GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
-          GST_AUDIO_CHANNEL_POSITION_REAR_CENTER},
     };
     size_t pair;
 
@@ -630,8 +561,13 @@ gst_opus_enc_setup_channel_mappings (GstOpusEnc * enc,
         GST_DEBUG_OBJECT (enc, "Channel position %s is not mapped yet, adding",
             gst_opus_channel_names[position]);
         cv = gst_opus_enc_find_channel_position_in_vorbis_order (enc, position);
-        if (cv < 0)
-          g_assert_not_reached ();
+        if (cv < 0) {
+          GST_WARNING_OBJECT (enc,
+              "Cannot map channel positions to Vorbis order, using unknown mapping");
+          enc->channel_mapping_family = 255;
+          enc->n_stereo_streams = 0;
+          return;
+        }
         enc->encoding_channel_mapping[mapped] = n;
         enc->decoding_channel_mapping[cv] = mapped;
         mapped++;
@@ -690,9 +626,6 @@ gst_opus_enc_set_format (GstAudioEncoder * benc, GstAudioInfo * info)
     return FALSE;
   }
 
-  /* update the tags */
-  gst_opus_enc_set_tags (enc);
-
   enc->frame_samples = gst_opus_enc_get_frame_samples (enc);
 
   /* feedback to base class */
@@ -707,12 +640,6 @@ static gboolean
 gst_opus_enc_setup (GstOpusEnc * enc)
 {
   int error = OPUS_OK;
-  GstCaps *caps;
-  gboolean ret;
-  gint32 lookahead;
-  const GstTagList *tags;
-  GstTagList *empty_tags = NULL;
-  GstBuffer *header, *comments;
 
 #ifndef GST_DISABLE_GST_DEBUG
   GST_DEBUG_OBJECT (enc,
@@ -730,18 +657,17 @@ gst_opus_enc_setup (GstOpusEnc * enc)
   enc->state = opus_multistream_encoder_create (enc->sample_rate,
       enc->n_channels, enc->n_channels - enc->n_stereo_streams,
       enc->n_stereo_streams, enc->encoding_channel_mapping,
-      enc->audio_type, &error);
+      enc->audio_or_voip ? OPUS_APPLICATION_AUDIO : OPUS_APPLICATION_VOIP,
+      &error);
   if (!enc->state || error != OPUS_OK)
     goto encoder_creation_failed;
 
   opus_multistream_encoder_ctl (enc->state, OPUS_SET_BITRATE (enc->bitrate), 0);
   opus_multistream_encoder_ctl (enc->state, OPUS_SET_BANDWIDTH (enc->bandwidth),
       0);
+  opus_multistream_encoder_ctl (enc->state, OPUS_SET_VBR (!enc->cbr), 0);
   opus_multistream_encoder_ctl (enc->state,
-      OPUS_SET_VBR (enc->bitrate_type != BITRATE_TYPE_CBR), 0);
-  opus_multistream_encoder_ctl (enc->state,
-      OPUS_SET_VBR_CONSTRAINT (enc->bitrate_type ==
-          BITRATE_TYPE_CONSTRAINED_VBR), 0);
+      OPUS_SET_VBR_CONSTRAINT (enc->constrained_vbr), 0);
   opus_multistream_encoder_ctl (enc->state,
       OPUS_SET_COMPLEXITY (enc->complexity), 0);
   opus_multistream_encoder_ctl (enc->state,
@@ -750,38 +676,9 @@ gst_opus_enc_setup (GstOpusEnc * enc)
   opus_multistream_encoder_ctl (enc->state,
       OPUS_SET_PACKET_LOSS_PERC (enc->packet_loss_percentage), 0);
 
-  opus_multistream_encoder_ctl (enc->state, OPUS_GET_LOOKAHEAD (&lookahead), 0);
+  GST_LOG_OBJECT (enc, "we have frame size %d", enc->frame_size);
 
-  GST_LOG_OBJECT (enc, "we have frame size %d, lookahead %d", enc->frame_size,
-      lookahead);
-
-  /* lookahead is samples, the Opus header wants it in 48kHz samples */
-  lookahead = lookahead * 48000 / enc->sample_rate;
-  enc->lookahead = enc->pending_lookahead = lookahead;
-
-  header = gst_codec_utils_opus_create_header (enc->sample_rate,
-      enc->n_channels, enc->channel_mapping_family,
-      enc->n_channels - enc->n_stereo_streams, enc->n_stereo_streams,
-      enc->decoding_channel_mapping, lookahead, 0);
-  tags = gst_tag_setter_get_tag_list (GST_TAG_SETTER (enc));
-  if (!tags)
-    tags = empty_tags = gst_tag_list_new_empty ();
-  comments =
-      gst_tag_list_to_vorbiscomment_buffer (tags, (const guint8 *) "OpusTags",
-      8, "Encoded with GStreamer opusenc");
-  caps = gst_codec_utils_opus_create_caps_from_header (header, comments);
-  if (empty_tags)
-    gst_tag_list_unref (empty_tags);
-  gst_buffer_unref (header);
-  gst_buffer_unref (comments);
-
-  /* negotiate with these caps */
-  GST_DEBUG_OBJECT (enc, "here are the caps: %" GST_PTR_FORMAT, caps);
-
-  ret = gst_audio_encoder_set_output_format (GST_AUDIO_ENCODER (enc), caps);
-  gst_caps_unref (caps);
-
-  return ret;
+  return TRUE;
 
 encoder_creation_failed:
   GST_ERROR_OBJECT (enc, "Encoder creation failed");
@@ -807,10 +704,6 @@ gst_opus_enc_sink_event (GstAudioEncoder * benc, GstEvent * event)
       gst_tag_setter_merge_tags (setter, list, mode);
       break;
     }
-    case GST_EVENT_SEGMENT:
-      enc->encoded_samples = 0;
-      enc->consumed_samples = 0;
-      break;
 
     default:
       break;
@@ -819,96 +712,72 @@ gst_opus_enc_sink_event (GstAudioEncoder * benc, GstEvent * event)
   return GST_AUDIO_ENCODER_CLASS (parent_class)->sink_event (benc, event);
 }
 
-static GstCaps *
-gst_opus_enc_get_sink_template_caps (void)
-{
-  static volatile gsize init = 0;
-  static GstCaps *caps = NULL;
-
-  if (g_once_init_enter (&init)) {
-    GValue rate_array = G_VALUE_INIT;
-    GValue v = G_VALUE_INIT;
-    GstStructure *s1, *s2, *s;
-    gint i, c;
-
-    caps = gst_caps_new_empty ();
-
-    /* Generate our two template structures */
-    g_value_init (&rate_array, GST_TYPE_LIST);
-    g_value_init (&v, G_TYPE_INT);
-    g_value_set_int (&v, 8000);
-    gst_value_list_append_value (&rate_array, &v);
-    g_value_set_int (&v, 12000);
-    gst_value_list_append_value (&rate_array, &v);
-    g_value_set_int (&v, 16000);
-    gst_value_list_append_value (&rate_array, &v);
-    g_value_set_int (&v, 24000);
-    gst_value_list_append_value (&rate_array, &v);
-
-    s1 = gst_structure_new ("audio/x-raw",
-        "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
-        "layout", G_TYPE_STRING, "interleaved",
-        "rate", G_TYPE_INT, 48000, NULL);
-    s2 = gst_structure_new ("audio/x-raw",
-        "format", G_TYPE_STRING, GST_AUDIO_NE (S16),
-        "layout", G_TYPE_STRING, "interleaved", NULL);
-    gst_structure_set_value (s2, "rate", &rate_array);
-    g_value_unset (&rate_array);
-    g_value_unset (&v);
-
-    /* Mono */
-    s = gst_structure_copy (s1);
-    gst_structure_set (s, "channels", G_TYPE_INT, 1, NULL);
-    gst_caps_append_structure (caps, s);
-
-    s = gst_structure_copy (s2);
-    gst_structure_set (s, "channels", G_TYPE_INT, 1, NULL);
-    gst_caps_append_structure (caps, s);
-
-    /* Stereo and further */
-    for (i = 2; i <= 8; i++) {
-      guint64 channel_mask = 0;
-      const GstAudioChannelPosition *pos = gst_opus_channel_positions[i - 1];
-
-      for (c = 0; c < i; c++) {
-        channel_mask |= G_GUINT64_CONSTANT (1) << pos[c];
-      }
-
-      s = gst_structure_copy (s1);
-      gst_structure_set (s, "channels", G_TYPE_INT, i, "channel-mask",
-          GST_TYPE_BITMASK, channel_mask, NULL);
-      gst_caps_append_structure (caps, s);
-
-      s = gst_structure_copy (s2);
-      gst_structure_set (s, "channels", G_TYPE_INT, i, "channel-mask",
-          GST_TYPE_BITMASK, channel_mask, NULL);
-      gst_caps_append_structure (caps, s);
-    }
-
-    gst_structure_free (s1);
-    gst_structure_free (s2);
-
-    g_once_init_leave (&init, 1);
-  }
-
-  return caps;
-}
-
 static GstCaps *
 gst_opus_enc_sink_getcaps (GstAudioEncoder * benc, GstCaps * filter)
 {
   GstOpusEnc *enc;
   GstCaps *caps;
+  GstCaps *peercaps = NULL;
+  GstCaps *intersect = NULL;
+  guint i;
+  gboolean allow_multistream;
 
   enc = GST_OPUS_ENC (benc);
 
   GST_DEBUG_OBJECT (enc, "sink getcaps");
 
-  caps = gst_opus_enc_get_sink_template_caps ();
-  caps = gst_audio_encoder_proxy_getcaps (benc, caps, filter);
+  peercaps = gst_pad_peer_query_caps (GST_AUDIO_ENCODER_SRC_PAD (benc), NULL);
+  if (!peercaps) {
+    GST_DEBUG_OBJECT (benc, "No peercaps, returning template sink caps");
+    return
+        gst_caps_copy (gst_pad_get_pad_template_caps
+        (GST_AUDIO_ENCODER_SINK_PAD (benc)));
+  }
+
+  intersect = gst_caps_intersect (peercaps,
+      gst_pad_get_pad_template_caps (GST_AUDIO_ENCODER_SRC_PAD (benc)));
+  gst_caps_unref (peercaps);
+
+  if (gst_caps_is_empty (intersect))
+    return intersect;
+
+  allow_multistream = FALSE;
+  for (i = 0; i < gst_caps_get_size (intersect); i++) {
+    GstStructure *s = gst_caps_get_structure (intersect, i);
+    gboolean multistream;
+    if (gst_structure_get_boolean (s, "multistream", &multistream)) {
+      if (multistream) {
+        allow_multistream = TRUE;
+      }
+    } else {
+      allow_multistream = TRUE;
+    }
+  }
+
+  gst_caps_unref (intersect);
+
+  caps =
+      gst_caps_copy (gst_pad_get_pad_template_caps (GST_AUDIO_ENCODER_SINK_PAD
+          (benc)));
+  if (!allow_multistream) {
+    GValue range = { 0 };
+    g_value_init (&range, GST_TYPE_INT_RANGE);
+    gst_value_set_int_range (&range, 1, 2);
+    for (i = 0; i < gst_caps_get_size (caps); i++) {
+      GstStructure *s = gst_caps_get_structure (caps, i);
+      gst_structure_set_value (s, "channels", &range);
+    }
+    g_value_unset (&range);
+  }
+
+  if (filter) {
+    GstCaps *tmp = gst_caps_intersect_full (caps, filter,
+        GST_CAPS_INTERSECT_FIRST);
+    gst_caps_unref (caps);
+    caps = tmp;
+  }
 
   GST_DEBUG_OBJECT (enc, "Returning caps: %" GST_PTR_FORMAT, caps);
-
   return caps;
 }
 
@@ -923,18 +792,15 @@ gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buf)
   GstMapInfo omap;
   gint outsize;
   GstBuffer *outbuf;
-  GstSegment *segment;
-  GstClockTime duration;
-  guint64 trim_start = 0, trim_end = 0;
 
   guint max_payload_size;
-  gint frame_samples, input_samples, output_samples;
+  gint frame_samples;
 
   g_mutex_lock (&enc->property_lock);
 
   bytes = enc->frame_samples * enc->n_channels * 2;
   max_payload_size = enc->max_payload_size;
-  frame_samples = input_samples = enc->frame_samples;
+  frame_samples = enc->frame_samples;
 
   g_mutex_unlock (&enc->property_lock);
 
@@ -944,120 +810,35 @@ gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buf)
     bsize = map.size;
 
     if (G_UNLIKELY (bsize % bytes)) {
-      gint64 diff;
-
       GST_DEBUG_OBJECT (enc, "draining; adding silence samples");
-      g_assert (bsize < bytes);
-
-      /* If encoding part of a frame, and we have no set stop time on
-       * the output segment, we update the segment stop time to reflect
-       * the last sample. This will let oggmux set the last page's
-       * granpos to tell a decoder the dummy samples should be clipped.
-       */
-      input_samples = bsize / (enc->n_channels * 2);
-      segment = &GST_AUDIO_ENCODER_OUTPUT_SEGMENT (enc);
-      if (!GST_CLOCK_TIME_IS_VALID (segment->stop)) {
-        GST_DEBUG_OBJECT (enc,
-            "No stop time and partial frame, updating segment");
-        duration =
-            gst_util_uint64_scale_ceil (enc->consumed_samples + input_samples,
-            GST_SECOND, enc->sample_rate);
-        segment->stop = segment->start + duration;
-        GST_DEBUG_OBJECT (enc, "new output segment %" GST_SEGMENT_FORMAT,
-            segment);
-        gst_pad_push_event (GST_AUDIO_ENCODER_SRC_PAD (enc),
-            gst_event_new_segment (segment));
-      }
-
-      diff =
-          (enc->encoded_samples + frame_samples) - (enc->consumed_samples +
-          input_samples);
-      if (diff >= 0) {
-        GST_DEBUG_OBJECT (enc,
-            "%" G_GINT64_FORMAT " extra samples of padding in this frame",
-            diff);
-        output_samples = frame_samples - diff;
-        trim_end = diff * 48000 / enc->sample_rate;
-      } else {
-        GST_DEBUG_OBJECT (enc,
-            "Need to add %" G_GINT64_FORMAT " extra samples in the next frame",
-            -diff);
-        output_samples = frame_samples;
-      }
 
       size = ((bsize / bytes) + 1) * bytes;
       mdata = g_malloc0 (size);
-      /* FIXME: Instead of silence, use LPC with the last real samples.
-       * Otherwise we will create a discontinuity here, which will distort the
-       * last few encoded samples
-       */
       memcpy (mdata, bdata, bsize);
       data = mdata;
     } else {
       data = bdata;
       size = bsize;
-
-      /* Adjust for lookahead here */
-      if (enc->pending_lookahead) {
-        guint scaled_lookahead =
-            enc->pending_lookahead * enc->sample_rate / 48000;
-
-        if (input_samples > scaled_lookahead) {
-          output_samples = input_samples - scaled_lookahead;
-          trim_start = enc->pending_lookahead;
-          enc->pending_lookahead = 0;
-        } else {
-          trim_start = ((guint64) input_samples) * 48000 / enc->sample_rate;
-          enc->pending_lookahead -= trim_start;
-          output_samples = 0;
-        }
-      } else {
-        output_samples = input_samples;
-      }
     }
   } else {
-    if (enc->encoded_samples < enc->consumed_samples) {
-      /* FIXME: Instead of silence, use LPC with the last real samples.
-       * Otherwise we will create a discontinuity here, which will distort the
-       * last few encoded samples
-       */
-      data = mdata = g_malloc0 (bytes);
-      size = bytes;
-      output_samples = enc->consumed_samples - enc->encoded_samples;
-      input_samples = 0;
-      GST_DEBUG_OBJECT (enc, "draining %d samples", output_samples);
-      trim_end =
-          ((guint64) frame_samples - output_samples) * 48000 / enc->sample_rate;
-    } else if (enc->encoded_samples == enc->consumed_samples) {
-      GST_DEBUG_OBJECT (enc, "nothing to drain");
-      goto done;
-    } else {
-      g_assert_not_reached ();
-      goto done;
-    }
+    GST_DEBUG_OBJECT (enc, "nothing to drain");
+    goto done;
   }
 
   g_assert (size == bytes);
 
-  outbuf =
-      gst_audio_encoder_allocate_output_buffer (GST_AUDIO_ENCODER (enc),
-      max_payload_size * enc->n_channels);
+  outbuf = gst_buffer_new_and_alloc (max_payload_size * enc->n_channels);
   if (!outbuf)
     goto done;
 
   GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)",
       frame_samples, (int) bytes);
 
-  if (trim_start || trim_end) {
-    GST_DEBUG_OBJECT (enc,
-        "Adding trim-start %" G_GUINT64_FORMAT " trim-end %" G_GUINT64_FORMAT,
-        trim_start, trim_end);
-    gst_buffer_add_audio_clipping_meta (outbuf, GST_FORMAT_DEFAULT, trim_start,
-        trim_end);
-  }
-
   gst_buffer_map (outbuf, &omap, GST_MAP_WRITE);
 
+  GST_DEBUG_OBJECT (enc, "encoding %d samples (%d bytes)",
+      frame_samples, (int) bytes);
+
   outsize =
       opus_multistream_encode (enc->state, (const gint16 *) data,
       frame_samples, omap.data, max_payload_size * enc->n_channels);
@@ -1079,19 +860,17 @@ gst_opus_enc_encode (GstOpusEnc * enc, GstBuffer * buf)
   GST_DEBUG_OBJECT (enc, "Output packet is %u bytes", outsize);
   gst_buffer_set_size (outbuf, outsize);
 
-
   ret =
       gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (enc), outbuf,
-      output_samples);
-  enc->encoded_samples += output_samples;
-  enc->consumed_samples += input_samples;
+      frame_samples);
 
 done:
 
   if (bdata)
     gst_buffer_unmap (buf, &map);
 
-  g_free (mdata);
+  if (mdata)
+    g_free (mdata);
 
   return ret;
 }
@@ -1104,6 +883,29 @@ gst_opus_enc_handle_frame (GstAudioEncoder * benc, GstBuffer * buf)
 
   enc = GST_OPUS_ENC (benc);
   GST_DEBUG_OBJECT (enc, "handle_frame");
+
+  if (!enc->header_sent) {
+    GstCaps *caps;
+
+    g_slist_foreach (enc->headers, (GFunc) gst_buffer_unref, NULL);
+    g_slist_free (enc->headers);
+    enc->headers = NULL;
+
+    gst_opus_header_create_caps (&caps, &enc->headers, enc->n_channels,
+        enc->n_stereo_streams, enc->sample_rate, enc->channel_mapping_family,
+        enc->decoding_channel_mapping,
+        gst_tag_setter_get_tag_list (GST_TAG_SETTER (enc)));
+
+
+    /* negotiate with these caps */
+    GST_DEBUG_OBJECT (enc, "here are the caps: %" GST_PTR_FORMAT, caps);
+
+    gst_audio_encoder_set_output_format (benc, caps);
+    gst_caps_unref (caps);
+
+    enc->header_sent = TRUE;
+  }
+
   GST_DEBUG_OBJECT (enc, "received buffer %p of %" G_GSIZE_FORMAT " bytes", buf,
       buf ? gst_buffer_get_size (buf) : 0);
 
@@ -1124,11 +926,7 @@ gst_opus_enc_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case PROP_AUDIO:
-      g_value_set_boolean (value,
-          enc->audio_type == OPUS_APPLICATION_AUDIO ? TRUE : FALSE);
-      break;
-    case PROP_AUDIO_TYPE:
-      g_value_set_enum (value, enc->audio_type);
+      g_value_set_boolean (value, enc->audio_or_voip);
       break;
     case PROP_BITRATE:
       g_value_set_int (value, enc->bitrate);
@@ -1140,18 +938,10 @@ gst_opus_enc_get_property (GObject * object, guint prop_id, GValue * value,
       g_value_set_enum (value, enc->frame_size);
       break;
     case PROP_CBR:
-      GST_WARNING_OBJECT (enc,
-          "cbr property is deprecated; use bitrate-type instead");
-      g_value_set_boolean (value, enc->bitrate_type == BITRATE_TYPE_CBR);
+      g_value_set_boolean (value, enc->cbr);
       break;
     case PROP_CONSTRAINED_VBR:
-      GST_WARNING_OBJECT (enc,
-          "constrained-vbr property is deprecated; use bitrate-type instead");
-      g_value_set_boolean (value,
-          enc->bitrate_type == BITRATE_TYPE_CONSTRAINED_VBR);
-      break;
-    case PROP_BITRATE_TYPE:
-      g_value_set_enum (value, enc->bitrate_type);
+      g_value_set_boolean (value, enc->constrained_vbr);
       break;
     case PROP_COMPLEXITY:
       g_value_set_int (value, enc->complexity);
@@ -1195,12 +985,7 @@ gst_opus_enc_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_AUDIO:
-      enc->audio_type =
-          g_value_get_boolean (value) ? OPUS_APPLICATION_AUDIO :
-          OPUS_APPLICATION_VOIP;
-      break;
-    case PROP_AUDIO_TYPE:
-      enc->audio_type = g_value_get_enum (value);
+      enc->audio_or_voip = g_value_get_boolean (value);
       break;
     case PROP_BITRATE:
       GST_OPUS_UPDATE_PROPERTY (bitrate, int, BITRATE);
@@ -1216,44 +1001,15 @@ gst_opus_enc_set_property (GObject * object, guint prop_id,
       g_mutex_unlock (&enc->property_lock);
       break;
     case PROP_CBR:
-      GST_WARNING_OBJECT (enc,
-          "cbr property is deprecated; use bitrate-type instead");
-      g_warning ("cbr property is deprecated; use bitrate-type instead");
+      /* this one has an opposite meaning to the opus ctl... */
       g_mutex_lock (&enc->property_lock);
-      enc->bitrate_type = BITRATE_TYPE_CBR;
-      if (enc->state) {
-        opus_multistream_encoder_ctl (enc->state, OPUS_SET_VBR (FALSE));
-        opus_multistream_encoder_ctl (enc->state,
-            OPUS_SET_VBR_CONSTRAINT (FALSE), 0);
-      }
+      enc->cbr = g_value_get_boolean (value);
+      if (enc->state)
+        opus_multistream_encoder_ctl (enc->state, OPUS_SET_VBR (!enc->cbr));
       g_mutex_unlock (&enc->property_lock);
       break;
     case PROP_CONSTRAINED_VBR:
-      GST_WARNING_OBJECT (enc,
-          "constrained-vbr property is deprecated; use bitrate-type instead");
-      g_warning
-          ("constrained-vbr property is deprecated; use bitrate-type instead");
-      g_mutex_lock (&enc->property_lock);
-      enc->bitrate_type = BITRATE_TYPE_CONSTRAINED_VBR;
-      if (enc->state) {
-        opus_multistream_encoder_ctl (enc->state, OPUS_SET_VBR (TRUE));
-        opus_multistream_encoder_ctl (enc->state,
-            OPUS_SET_VBR_CONSTRAINT (TRUE), 0);
-      }
-      g_mutex_unlock (&enc->property_lock);
-      break;
-    case PROP_BITRATE_TYPE:
-      /* this one has an opposite meaning to the opus ctl... */
-      g_mutex_lock (&enc->property_lock);
-      enc->bitrate_type = g_value_get_enum (value);
-      if (enc->state) {
-        opus_multistream_encoder_ctl (enc->state,
-            OPUS_SET_VBR (enc->bitrate_type != BITRATE_TYPE_CBR));
-        opus_multistream_encoder_ctl (enc->state,
-            OPUS_SET_VBR_CONSTRAINT (enc->bitrate_type ==
-                BITRATE_TYPE_CONSTRAINED_VBR), 0);
-      }
-      g_mutex_unlock (&enc->property_lock);
+      GST_OPUS_UPDATE_PROPERTY (constrained_vbr, boolean, VBR_CONSTRAINT);
       break;
     case PROP_COMPLEXITY:
       GST_OPUS_UPDATE_PROPERTY (complexity, int, COMPLEXITY);
