@@ -80,7 +80,7 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
   GstSbcEnc *enc = GST_SBC_ENC (audio_enc);
   GstStructure *s;
   GstCaps *caps, *filter_caps;
-  GstCaps *output_caps = NULL;
+  GstCaps *output_caps;
   guint sampleframes_per_frame;
 
   enc->rate = GST_AUDIO_INFO_RATE (info);
@@ -88,8 +88,10 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
 
   /* negotiate output format based on downstream caps restrictions */
   caps = gst_pad_get_allowed_caps (GST_AUDIO_ENCODER_SRC_PAD (enc));
-  if (caps == GST_CAPS_NONE || gst_caps_is_empty (caps))
-    goto failure;
+  if (caps == GST_CAPS_NONE || gst_caps_is_empty (caps)) {
+    gst_caps_unref (caps);
+    return FALSE;
+  }
 
   if (caps == NULL)
     caps = gst_static_pad_template_get_caps (&sbc_enc_src_factory);
@@ -104,7 +106,10 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
     GST_WARNING_OBJECT (enc, "Couldn't negotiate output caps with input rate "
         "%d and input channels %d and allowed output caps %" GST_PTR_FORMAT,
         enc->rate, enc->channels, caps);
-    goto failure;
+    if (output_caps)
+      gst_caps_unref (output_caps);
+    gst_caps_unref (caps);
+    return FALSE;
   }
 
   gst_caps_unref (caps);
@@ -143,7 +148,8 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
     if (g_strcmp0 (channel_mode, "mono") != 0) {
       GST_ERROR_OBJECT (enc, "Can't have channel-mode '%s' for 1 channel",
           channel_mode);
-      goto failure;
+      gst_caps_unref (output_caps);
+      return FALSE;
     }
   } else {
     if (g_strcmp0 (channel_mode, "joint") != 0 &&
@@ -151,7 +157,8 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
         g_strcmp0 (channel_mode, "dual") != 0) {
       GST_ERROR_OBJECT (enc, "Can't have channel-mode '%s' for 2 channels",
           channel_mode);
-      goto failure;
+      gst_caps_unref (output_caps);
+      return FALSE;
     }
   }
 
@@ -175,7 +182,7 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
   else if (enc->rate == 48000)
     enc->sbc.frequency = SBC_FREQ_48000;
   else
-    goto failure;
+    return FALSE;
 
   if (enc->blocks == 4)
     enc->sbc.blocks = SBC_BLK_4;
@@ -186,13 +193,13 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
   else if (enc->blocks == 16)
     enc->sbc.blocks = SBC_BLK_16;
   else
-    goto failure;
+    return FALSE;
 
   enc->sbc.subbands = (enc->subbands == 4) ? SBC_SB_4 : SBC_SB_8;
   enc->sbc.bitpool = enc->bitpool;
 
   if (channel_mode == NULL || allocation_method == NULL)
-    goto failure;
+    return FALSE;
 
   if (strcmp (channel_mode, "joint") == 0)
     enc->sbc.mode = SBC_MODE_JOINT_STEREO;
@@ -205,26 +212,19 @@ gst_sbc_enc_set_format (GstAudioEncoder * audio_enc, GstAudioInfo * info)
   else if (strcmp (channel_mode, "auto") == 0)
     enc->sbc.mode = SBC_MODE_JOINT_STEREO;
   else
-    goto failure;
+    return FALSE;
 
   if (strcmp (allocation_method, "loudness") == 0)
     enc->sbc.allocation = SBC_AM_LOUDNESS;
   else if (strcmp (allocation_method, "snr") == 0)
     enc->sbc.allocation = SBC_AM_SNR;
   else
-    goto failure;
+    return FALSE;
 
   if (!gst_audio_encoder_set_output_format (audio_enc, output_caps))
-    goto failure;
+    return FALSE;
 
   return gst_audio_encoder_negotiate (audio_enc);
-
-failure:
-  if (output_caps)
-    gst_caps_unref (output_caps);
-  if (caps)
-    gst_caps_unref (caps);
-  return FALSE;
 }
 
 static GstFlowReturn
@@ -346,10 +346,11 @@ gst_sbc_enc_class_init (GstSbcEncClass * klass)
   encoder_class->set_format = GST_DEBUG_FUNCPTR (gst_sbc_enc_set_format);
   encoder_class->handle_frame = GST_DEBUG_FUNCPTR (gst_sbc_enc_handle_frame);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &sbc_enc_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &sbc_enc_src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sbc_enc_sink_factory));
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sbc_enc_src_factory));
 
   gst_element_class_set_static_metadata (element_class,
       "Bluetooth SBC audio encoder", "Codec/Encoder/Audio",
@@ -361,7 +362,6 @@ gst_sbc_enc_class_init (GstSbcEncClass * klass)
 static void
 gst_sbc_enc_init (GstSbcEnc * self)
 {
-  GST_PAD_SET_ACCEPT_TEMPLATE (GST_AUDIO_ENCODER_SINK_PAD (self));
   self->subbands = 0;
   self->blocks = 0;
   self->rate = 0;
