@@ -23,41 +23,47 @@
 
 /**
  * SECTION:element-aiffparse
- * @title: aiffparse
  *
+ * <refsect2>
+ * <para>
  * Parse a .aiff file into raw or compressed audio.
- *
+ * </para>
+ * <para>
  * The aiffparse element supports both push and pull mode operations, making it
  * possible to stream from a network source.
- *
- * ## Example launch line
- *
- * |[
- * gst-launch-1.0 filesrc location=sine.aiff ! aiffparse ! audioconvert ! alsasink
- * ]|
+ * </para>
+ * <title>Example launch line</title>
+ * <para>
+ * <programlisting>
+ * gst-launch filesrc location=sine.aiff ! aiffparse ! audioconvert ! alsasink
+ * </programlisting>
  * Read a aiff file and output to the soundcard using the ALSA element. The
  * aiff file is assumed to contain raw uncompressed samples.
- *
- * |[
- * gst-launch-1.0 souphttpsrc location=http://www.example.org/sine.aiff ! queue ! aiffparse ! audioconvert ! alsasink
- * ]|
+ * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch souphhtpsrc location=http://www.example.org/sine.aiff ! queue ! aiffparse ! audioconvert ! alsasink
+ * </programlisting>
  * Stream data from a network url.
- *
+ * </para>
+ * </refsect2>
  */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <string.h>
 #include <math.h>
 
-#include "aiffelements.h"
 #include "aiffparse.h"
 #include <gst/audio/audio.h>
 #include <gst/tag/tag.h>
-#include <gst/pbutils/descriptions.h>
-#include <glib/gi18n-lib.h>
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY (aiffparse_debug);
 #define GST_CAT_DEFAULT (aiffparse_debug)
@@ -105,10 +111,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
 #define MAX_BUFFER_SIZE 4096
 
 #define gst_aiff_parse_parent_class parent_class
-G_DEFINE_TYPE_WITH_CODE (GstAiffParse, gst_aiff_parse, GST_TYPE_ELEMENT,
-    GST_DEBUG_CATEGORY_INIT (aiffparse_debug, "aiffparse", 0, "AIFF parser"));
-GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (aiffparse, "aiffparse", GST_RANK_PRIMARY,
-    GST_TYPE_AIFF_PARSE, aiff_element_init (plugin));
+G_DEFINE_TYPE (GstAiffParse, gst_aiff_parse, GST_TYPE_ELEMENT);
 
 static void
 gst_aiff_parse_class_init (GstAiffParseClass * klass)
@@ -121,10 +124,10 @@ gst_aiff_parse_class_init (GstAiffParseClass * klass)
 
   object_class->dispose = gst_aiff_parse_dispose;
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &sink_template_factory);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &src_template_factory);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&src_template_factory));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "AIFF audio demuxer", "Codec/Demuxer/Audio",
@@ -647,17 +650,14 @@ gst_aiff_parse_calculate_duration (GstAiffParse * aiff)
   return FALSE;
 }
 
-static gboolean
+static void
 gst_aiff_parse_ignore_chunk (GstAiffParse * aiff, guint32 tag, guint32 size)
 {
   guint flush;
 
   if (aiff->streaming) {
-    if (!gst_aiff_parse_peek_chunk (aiff, &tag, &size)) {
-      GST_LOG_OBJECT (aiff, "Not enough data to skip tag %" GST_FOURCC_FORMAT,
-          GST_FOURCC_ARGS (tag));
-      return FALSE;
-    }
+    if (!gst_aiff_parse_peek_chunk (aiff, &tag, &size))
+      return;
   }
   GST_WARNING_OBJECT (aiff, "Ignoring tag %" GST_FOURCC_FORMAT,
       GST_FOURCC_ARGS (tag));
@@ -666,7 +666,6 @@ gst_aiff_parse_ignore_chunk (GstAiffParse * aiff, guint32 tag, guint32 size)
   if (aiff->streaming) {
     gst_adapter_flush (aiff->adapter, flush);
   }
-  return TRUE;
 }
 
 static double
@@ -821,20 +820,17 @@ too_small:
 
 }
 
-#define _P(pos) (G_GUINT64_CONSTANT (1) << GST_AUDIO_CHANNEL_POSITION_ ##pos)
-
 static GstCaps *
 gst_aiff_parse_create_caps (GstAiffParse * aiff)
 {
   GstCaps *caps = NULL;
   const gchar *format = NULL;
-  guint64 channel_mask = 0;
 
   if (aiff->floating_point) {
     if (aiff->endianness == G_BIG_ENDIAN) {
       if (aiff->width == 32)
         format = "F32BE";
-      else if (aiff->width == 64)
+      else if (aiff->width == 32)
         format = "F64BE";
     }
   } else {
@@ -866,39 +862,6 @@ gst_aiff_parse_create_caps (GstAiffParse * aiff)
         "rate", G_TYPE_INT, aiff->rate, NULL);
   }
 
-  /* based on AIFF-1.3.pdf */
-  switch (aiff->channels) {
-    case 1:
-      channel_mask = 0;
-      break;
-    case 2:
-      channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT);
-      break;
-    case 3:
-      channel_mask = _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (FRONT_CENTER);
-      break;
-    case 4:
-      /* lists both this and 'quad' but doesn't say how to distinguish the two */
-      channel_mask =
-          _P (FRONT_LEFT) | _P (FRONT_RIGHT) | _P (REAR_LEFT) | _P (REAR_RIGHT);
-      break;
-    case 6:
-      channel_mask =
-          _P (FRONT_LEFT) | _P (FRONT_LEFT_OF_CENTER) | _P (FRONT_CENTER) |
-          _P (FRONT_RIGHT) | _P (FRONT_RIGHT_OF_CENTER) | _P (LFE1);
-      break;
-    default:
-      GST_FIXME_OBJECT (aiff, "using fallback channel layout for %d channels",
-          aiff->channels);
-      channel_mask = gst_audio_channel_get_fallback_mask (aiff->channels);
-      break;
-  }
-
-  if (channel_mask != 0) {
-    gst_caps_set_simple (caps, "channel-mask", GST_TYPE_BITMASK, channel_mask,
-        NULL);
-  }
-
   GST_DEBUG_OBJECT (aiff, "Created caps: %" GST_PTR_FORMAT, caps);
 
   return caps;
@@ -995,32 +958,6 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
         aiff->bytes_per_sample = aiff->channels * aiff->width / 8;
         aiff->bps = aiff->bytes_per_sample * aiff->rate;
 
-        if (!aiff->tags)
-          aiff->tags = gst_tag_list_new_empty ();
-
-        {
-          GstCaps *templ_caps = gst_pad_get_pad_template_caps (aiff->sinkpad);
-          gst_pb_utils_add_codec_description_to_tag_list (aiff->tags,
-              GST_TAG_CONTAINER_FORMAT, templ_caps);
-          gst_caps_unref (templ_caps);
-        }
-
-        if (aiff->bps) {
-          guint bitrate = aiff->bps * 8;
-
-          GST_DEBUG_OBJECT (aiff, "adding bitrate of %u bps to tag list",
-              bitrate);
-
-          /* At the moment, aiffparse only supports uncompressed PCM data.
-           * Therefore, nominal, actual, minimum, maximum bitrate are the same.
-           * XXX: If AIFF-C support is extended to include compression,
-           * make sure that aiff->bps is set properly. */
-          gst_tag_list_add (aiff->tags, GST_TAG_MERGE_REPLACE,
-              GST_TAG_BITRATE, bitrate, GST_TAG_NOMINAL_BITRATE, bitrate,
-              GST_TAG_MINIMUM_BITRATE, bitrate, GST_TAG_MAXIMUM_BITRATE,
-              bitrate, NULL);
-        }
-
         if (aiff->bytes_per_sample <= 0)
           goto no_bytes_per_sample;
 
@@ -1117,17 +1054,8 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
         }
         break;
       }
-      case GST_MAKE_FOURCC ('C', 'H', 'A', 'N'):{
-        GST_FIXME_OBJECT (aiff, "Handle CHAN chunk with channel layouts");
-        if (!gst_aiff_parse_ignore_chunk (aiff, tag, size)) {
-          return GST_FLOW_OK;
-        }
-        break;
-      }
       default:
-        if (!gst_aiff_parse_ignore_chunk (aiff, tag, size)) {
-          return GST_FLOW_OK;
-        }
+        gst_aiff_parse_ignore_chunk (aiff, tag, size);
     }
 
     buf = NULL;
@@ -1530,7 +1458,9 @@ pause:
     } else if (ret < GST_FLOW_EOS || ret == GST_FLOW_NOT_LINKED) {
       /* for fatal errors we post an error message, post the error
        * first so the app knows about the error first. */
-      GST_ELEMENT_FLOW_ERROR (aiff, ret);
+      GST_ELEMENT_ERROR (aiff, STREAM, FAILED,
+          (_("Internal data flow error.")),
+          ("streaming task paused, reason %s (%d)", reason, ret));
       gst_pad_push_event (aiff->srcpad, gst_event_new_eos ());
     }
     return;
@@ -1557,7 +1487,7 @@ gst_aiff_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
       if (aiff->state != AIFF_PARSE_HEADER)
         break;
 
-      G_GNUC_FALLTHROUGH;
+      /* otherwise fall-through */
     case AIFF_PARSE_HEADER:
       GST_INFO_OBJECT (aiff, "AIFF_PARSE_HEADER");
       if ((ret = gst_aiff_parse_stream_headers (aiff)) != GST_FLOW_OK)
@@ -1569,7 +1499,7 @@ gst_aiff_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
       aiff->state = AIFF_PARSE_DATA;
       GST_INFO_OBJECT (aiff, "AIFF_PARSE_DATA");
 
-      /* FALLTHROUGH */
+      /* fall-through */
     case AIFF_PARSE_DATA:
       if ((ret = gst_aiff_parse_stream_data (aiff)) != GST_FLOW_OK)
         goto done;
@@ -1892,13 +1822,15 @@ gst_aiff_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
         }
         if (stop > 0) {
           end_offset = stop;
-          stop -= aiff->datastart;
-          stop = MAX (stop, 0);
+          segment.stop -= aiff->datastart;
+          segment.stop = MAX (stop, 0);
         }
         if (aiff->state == AIFF_PARSE_DATA &&
             aiff->segment.format == GST_FORMAT_TIME) {
+          guint64 bps = aiff->bps;
+
           /* operating in format TIME, so we can convert */
-          if (aiff->bps) {
+          if (bps) {
             if (start >= 0)
               start =
                   gst_util_uint64_scale_ceil (start, GST_SECOND,
@@ -1934,15 +1866,6 @@ gst_aiff_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 
       aiff->start_segment = gst_event_new_segment (&segment);
 
-      /* If the seek is within the same SSND chunk and there is no new
-       * end_offset defined keep the previous end_offset. This will avoid noise
-       * at the end of playback if e.g. a metadata chunk is located at the end
-       * of the file. */
-      if (aiff->end_offset > 0 && offset < aiff->end_offset &&
-          offset >= aiff->datastart && end_offset == -1) {
-        end_offset = aiff->end_offset;
-      }
-
       /* stream leftover data in current segment */
       if (aiff->state == AIFF_PARSE_DATA)
         gst_aiff_parse_flush_data (aiff);
