@@ -40,7 +40,7 @@
 #define MPEGTS_MAX_PACKETSIZE MPEGTS_ATSC_PACKETSIZE
 
 #define MPEGTS_AFC_DISCONTINUITY_FLAG           0x80
-#define MPEGTS_AFC_RANDOM_ACCESS_FLAG           0x40
+#define MPEGTS_AFC_RANDOM_ACCES_FLAGS           0x40
 #define MPEGTS_AFC_ELEMENTARY_STREAM_PRIORITY   0x20
 #define MPEGTS_AFC_PCR_FLAG                     0x10
 #define MPEGTS_AFC_OPCR_FLAG                    0x08
@@ -99,10 +99,10 @@ typedef struct
 typedef struct _PCROffset
 {
   /* PCR value (units: 1/27MHz) */
-  guint64 pcr;
+  guint32 pcr;
 
   /* The offset (units: bytes) */
-  guint64 offset;
+  guint32 offset;
 } PCROffset;
 
 /* Flags used on groups */
@@ -259,8 +259,6 @@ struct _MpegTSPacketizer2 {
 
   /* clock skew calculation */
   gboolean       calculate_skew;
-  /* skew_correction: apply skew correction to values */
-  gboolean       skew_correction;
 
   /* offset/bitrate calculator */
   gboolean       calculate_offset;
@@ -284,15 +282,6 @@ struct _MpegTSPacketizer2 {
   guint8 pcrtablelut[0x2000];
   MpegTSPCR *observations[MAX_PCR_OBS_CHANNELS];
   guint8 lastobsid;
-  GstClockTime pcr_discont_threshold;
-
-  /* PTS/DTS of last buffer */
-  GstClockTime last_pts;
-  GstClockTime last_dts;
-
-  /* Extra time offset to handle values before initial PCR.
-   * This will be added to all converted timestamps */
-  GstClockTime extra_shift;
 };
 
 struct _MpegTSPacketizer2Class {
@@ -311,7 +300,6 @@ typedef struct
   guint8  scram_afc_cc;
   guint8 *payload;
 
-  guint8 *m2ts_header_start;
   guint8 *data_start;
   guint8 *data_end;
   guint8 *data;
@@ -366,6 +354,8 @@ G_GNUC_INTERNAL GstMpegtsSection *mpegts_packetizer_push_section (MpegTSPacketiz
 								  MpegTSPacketizerPacket *packet, GList **remaining);
 
 /* Only valid if calculate_offset is TRUE */
+G_GNUC_INTERNAL guint mpegts_packetizer_get_seen_pcr (MpegTSPacketizer2 *packetizer);
+
 G_GNUC_INTERNAL GstClockTime
 mpegts_packetizer_offset_to_ts (MpegTSPacketizer2 * packetizer,
 				guint64 offset, guint16 pcr_pid);
@@ -375,21 +365,12 @@ mpegts_packetizer_ts_to_offset (MpegTSPacketizer2 * packetizer,
 G_GNUC_INTERNAL GstClockTime
 mpegts_packetizer_pts_to_ts (MpegTSPacketizer2 * packetizer,
 			     GstClockTime pts, guint16 pcr_pid);
-G_GNUC_INTERNAL GstClockTime
-mpegts_packetizer_pts_to_ts_unchecked (MpegTSPacketizer2 * packetizer,
-			     GstClockTime pts, guint16 pcr_pid);
-G_GNUC_INTERNAL GstClockTime
-mpegts_packetizer_get_current_time (MpegTSPacketizer2 * packetizer,
-				    guint16 pcr_pid);
 G_GNUC_INTERNAL void
 mpegts_packetizer_set_current_pcr_offset (MpegTSPacketizer2 * packetizer,
 			  GstClockTime offset, guint16 pcr_pid);
 G_GNUC_INTERNAL void
 mpegts_packetizer_set_reference_offset (MpegTSPacketizer2 * packetizer,
 					guint64 refoffset);
-G_GNUC_INTERNAL void
-mpegts_packetizer_set_pcr_discont_threshold (MpegTSPacketizer2 * packetizer,
-					GstClockTime threshold);
 G_END_DECLS
 
 #endif /* GST_MPEGTS_PACKETIZER_H */
