@@ -10,8 +10,6 @@
  * Copyright (C) 2013, Lemote Ltd.
  *   Author: Chen Jie <chenj@lemote.com>
  *
- * Copyright (C) 2015, Sebastian Dröge <sebastian@centricular.com>
- *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation
@@ -41,10 +39,6 @@
 #define orc_memcpy memcpy
 #endif
 
-#ifdef HAVE_JNI_H
-#include "gstjniutils.h"
-#endif
-
 #include "gstamcvideoenc.h"
 #include "gstamc-constants.h"
 
@@ -99,8 +93,7 @@ enum
 {
   PROP_0,
   PROP_BIT_RATE,
-  PROP_I_FRAME_INTERVAL,
-  PROP_I_FRAME_INTERVAL_FLOAT
+  PROP_I_FRAME_INTERVAL
 };
 
 /* class initialization */
@@ -114,7 +107,7 @@ static GstVideoEncoderClass *parent_class = NULL;
 GType
 gst_amc_video_enc_get_type (void)
 {
-  static gsize type = 0;
+  static volatile gsize type = 0;
 
   if (g_once_init_enter (&type)) {
     GType _type;
@@ -190,7 +183,7 @@ create_amc_format (GstAmcVideoEnc * encoder, GstVideoCodecState * input_state,
 
       if (profile_string) {
         amc_profile.key = "profile";    /* named profile ? */
-        amc_profile.id = gst_amc_mpeg4_profile_from_string (profile_string);
+        amc_profile.id = gst_amc_avc_mpeg4_profile_from_string (profile_string);
       }
 
       if (level_string) {
@@ -213,25 +206,8 @@ create_amc_format (GstAmcVideoEnc * encoder, GstVideoCodecState * input_state,
       amc_level.key = "level";  /* named level ? */
       amc_level.id = gst_amc_avc_level_from_string (level_string);
     }
-  } else if (strcmp (name, "video/x-h265") == 0) {
-    const gchar *tier_string = gst_structure_get_string (s, "tier");
-
-    mime = "video/hevc";
-
-    if (profile_string) {
-      amc_profile.key = "profile";      /* named profile ? */
-      amc_profile.id = gst_amc_hevc_profile_from_string (profile_string);
-    }
-
-    if (level_string && tier_string) {
-      amc_level.key = "level";  /* named level ? */
-      amc_level.id =
-          gst_amc_hevc_tier_level_from_string (tier_string, level_string);
-    }
   } else if (strcmp (name, "video/x-vp8") == 0) {
     mime = "video/x-vnd.on2.vp8";
-  } else if (strcmp (name, "video/x-vp9") == 0) {
-    mime = "video/x-vnd.on2.vp9";
   } else {
     GST_ERROR_OBJECT (encoder, "Failed to convert caps(%s/...) to any mime",
         name);
@@ -283,22 +259,9 @@ create_amc_format (GstAmcVideoEnc * encoder, GstVideoCodecState * input_state,
     /* gst_amc_format_set_int (format, amc_level.key, amc_level.id); */
   }
 
-  /* On Android N_MR1 and higher, i-frame-interval can be a float value */
-#ifdef HAVE_JNI_H
-  if (gst_amc_jni_get_android_level () >= 25) {
-    GST_LOG_OBJECT (encoder, "Setting i-frame-interval to %f",
-        encoder->i_frame_int);
-    gst_amc_format_set_float (format, "i-frame-interval", encoder->i_frame_int,
+  if (encoder->i_frame_int)
+    gst_amc_format_set_int (format, "i-frame-interval", encoder->i_frame_int,
         &err);
-  } else
-#endif
-  {
-    int i_frame_int = encoder->i_frame_int;
-    /* Round a fractional interval to 1 per sec on older Android */
-    if (encoder->i_frame_int > 0 && encoder->i_frame_int < 1.0)
-      i_frame_int = 1;
-    gst_amc_format_set_int (format, "i-frame-interval", i_frame_int, &err);
-  }
   if (err)
     GST_ELEMENT_WARNING_FROM_ERROR (encoder, err);
 
@@ -339,12 +302,12 @@ color_format_info_failed_to_set:
   return NULL;
 
 unsupported_profile:
-  GST_ERROR_OBJECT (encoder, "Unsupported profile '%s'", profile_string);
+  GST_ERROR_OBJECT (encoder, "Unsupport profile '%s'", profile_string);
   gst_amc_format_free (format);
   return NULL;
 
 unsupported_level:
-  GST_ERROR_OBJECT (encoder, "Unsupported level '%s'", level_string);
+  GST_ERROR_OBJECT (encoder, "Unsupport level '%s'", level_string);
   gst_amc_format_free (format);
   return NULL;
 }
@@ -383,7 +346,8 @@ caps_from_amc_format (GstAmcFormat * amc_format)
 
     caps =
         gst_caps_new_simple ("video/mpeg", "mpegversion", G_TYPE_INT, 4,
-        "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
+        "systemstream", G_TYPE_BOOLEAN, FALSE,
+        "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
 
     if (gst_amc_format_get_int (amc_format, "profile", &amc_profile, NULL)) {
       profile_string = gst_amc_mpeg4_profile_to_string (amc_profile);
@@ -410,8 +374,9 @@ caps_from_amc_format (GstAmcFormat * amc_format)
     const gchar *profile_string, *level_string;
 
     caps =
-        gst_caps_new_simple ("video/x-h264",
-        "stream-format", G_TYPE_STRING, "byte-stream", NULL);
+        gst_caps_new_simple ("video/x-h264", "parsed", G_TYPE_BOOLEAN, TRUE,
+        "stream-format", G_TYPE_STRING, "byte-stream",
+        "alignment", G_TYPE_STRING, "au", NULL);
 
     if (gst_amc_format_get_int (amc_format, "profile", &amc_profile, NULL)) {
       profile_string = gst_amc_avc_profile_to_string (amc_profile, NULL);
@@ -429,36 +394,8 @@ caps_from_amc_format (GstAmcFormat * amc_format)
 
       gst_caps_set_simple (caps, "level", G_TYPE_STRING, level_string, NULL);
     }
-  } else if (strcmp (mime, "video/hevc") == 0) {
-    const gchar *profile_string, *level_string, *tier_string;
-
-    caps =
-        gst_caps_new_simple ("video/x-h265",
-        "stream-format", G_TYPE_STRING, "byte-stream", NULL);
-
-    if (gst_amc_format_get_int (amc_format, "profile", &amc_profile, NULL)) {
-      profile_string = gst_amc_avc_profile_to_string (amc_profile, NULL);
-      if (!profile_string)
-        goto unsupported_profile;
-
-      gst_caps_set_simple (caps, "profile", G_TYPE_STRING, profile_string,
-          NULL);
-    }
-
-    if (gst_amc_format_get_int (amc_format, "level", &amc_level, NULL)) {
-      level_string =
-          gst_amc_hevc_tier_level_to_string (amc_profile, &tier_string);
-      if (!level_string || !tier_string)
-        goto unsupported_level;
-
-      gst_caps_set_simple (caps,
-          "level", G_TYPE_STRING, level_string,
-          "tier", G_TYPE_STRING, tier_string, NULL);
-    }
   } else if (strcmp (mime, "video/x-vnd.on2.vp8") == 0) {
     caps = gst_caps_new_empty_simple ("video/x-vp8");
-  } else if (strcmp (mime, "video/x-vnd.on2.vp9") == 0) {
-    caps = gst_caps_new_empty_simple ("video/x-vp9");
   }
 
   gst_caps_set_simple (caps, "width", G_TYPE_INT, width,
@@ -469,16 +406,16 @@ caps_from_amc_format (GstAmcFormat * amc_format)
   return caps;
 
 unsupported_profile:
-  GST_ERROR ("Unsupported amc profile id %d", amc_profile);
+  GST_ERROR ("Unsupport amc profile id %d", amc_profile);
   g_free (mime);
-  gst_caps_unref (caps);
+  gst_object_unref (caps);
 
   return NULL;
 
 unsupported_level:
-  GST_ERROR ("Unsupported amc level id %d", amc_level);
+  GST_ERROR ("Unsupport amc level id %d", amc_level);
   g_free (mime);
-  gst_caps_unref (caps);
+  gst_object_unref (caps);
 
   return NULL;
 }
@@ -515,7 +452,7 @@ gst_amc_video_enc_base_init (gpointer g_class)
   longname = g_strdup_printf ("Android MediaCodec %s", codec_info->name);
   gst_element_class_set_metadata (element_class,
       codec_info->name,
-      "Codec/Encoder/Video/Hardware",
+      "Codec/Encoder/Video",
       longname, "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
   g_free (longname);
 }
@@ -526,45 +463,21 @@ gst_amc_video_enc_set_property (GObject * object, guint prop_id,
 {
   GstAmcVideoEnc *encoder;
   GstState state;
-  gboolean codec_active;
-  GError *err = NULL;
 
   encoder = GST_AMC_VIDEO_ENC (object);
 
   GST_OBJECT_LOCK (encoder);
 
   state = GST_STATE (encoder);
-  codec_active = (encoder->codec && state != GST_STATE_READY
-      && state != GST_STATE_NULL);
+  if (state != GST_STATE_READY && state != GST_STATE_NULL)
+    goto wrong_state;
 
   switch (prop_id) {
     case PROP_BIT_RATE:
       encoder->bitrate = g_value_get_uint (value);
-
-      g_mutex_lock (&encoder->codec_lock);
-      if (encoder->codec) {
-        if (!gst_amc_codec_set_dynamic_bitrate (encoder->codec, &err,
-                encoder->bitrate)) {
-          g_mutex_unlock (&encoder->codec_lock);
-          goto wrong_state;
-        }
-      }
-      g_mutex_unlock (&encoder->codec_lock);
-      if (err) {
-        GST_ELEMENT_WARNING_FROM_ERROR (encoder, err);
-        g_clear_error (&err);
-      }
-
       break;
     case PROP_I_FRAME_INTERVAL:
       encoder->i_frame_int = g_value_get_uint (value);
-      if (codec_active)
-        goto wrong_state;
-      break;
-    case PROP_I_FRAME_INTERVAL_FLOAT:
-      encoder->i_frame_int = g_value_get_float (value);
-      if (codec_active)
-        goto wrong_state;
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -597,9 +510,6 @@ gst_amc_video_enc_get_property (GObject * object, guint prop_id,
     case PROP_I_FRAME_INTERVAL:
       g_value_set_uint (value, encoder->i_frame_int);
       break;
-    case PROP_I_FRAME_INTERVAL_FLOAT:
-      g_value_set_float (value, encoder->i_frame_int);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -614,7 +524,6 @@ gst_amc_video_enc_class_init (GstAmcVideoEncClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
   GstVideoEncoderClass *videoenc_class = GST_VIDEO_ENCODER_CLASS (klass);
-  GParamFlags dynamic_flag = 0;
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -635,34 +544,21 @@ gst_amc_video_enc_class_init (GstAmcVideoEncClass * klass)
       GST_DEBUG_FUNCPTR (gst_amc_video_enc_handle_frame);
   videoenc_class->finish = GST_DEBUG_FUNCPTR (gst_amc_video_enc_finish);
 
-  // On Android >= 19, we can set bitrate dynamically
-  // so add the flag so apps can detect it.
-  if (gst_amc_codec_have_dynamic_bitrate ())
-    dynamic_flag = GST_PARAM_MUTABLE_PLAYING;
-
   g_object_class_install_property (gobject_class, PROP_BIT_RATE,
       g_param_spec_uint ("bitrate", "Bitrate", "Bitrate in bit/sec", 1,
           G_MAXINT, BIT_RATE_DEFAULT,
-          dynamic_flag | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_I_FRAME_INTERVAL,
       g_param_spec_uint ("i-frame-interval", "I-frame interval",
           "The frequency of I frames expressed in seconds between I frames (0 for automatic)",
           0, G_MAXINT, I_FRAME_INTERVAL_DEFAULT,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-
-  g_object_class_install_property (gobject_class, PROP_I_FRAME_INTERVAL_FLOAT,
-      g_param_spec_float ("i-frame-interval-float", "I-frame interval",
-          "The frequency of I frames expressed in seconds between I frames (0 for automatic). "
-          "Fractional intervals work on Android >= 25",
-          0, G_MAXFLOAT, I_FRAME_INTERVAL_DEFAULT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
 gst_amc_video_enc_init (GstAmcVideoEnc * self)
 {
-  g_mutex_init (&self->codec_lock);
   g_mutex_init (&self->drain_lock);
   g_cond_init (&self->drain_cond);
 
@@ -679,14 +575,11 @@ gst_amc_video_enc_open (GstVideoEncoder * encoder)
 
   GST_DEBUG_OBJECT (self, "Opening encoder");
 
-  g_mutex_lock (&self->codec_lock);
-  self->codec = gst_amc_codec_new (klass->codec_info->name, TRUE, &err);
+  self->codec = gst_amc_codec_new (klass->codec_info->name, &err);
   if (!self->codec) {
-    g_mutex_unlock (&self->codec_lock);
     GST_ELEMENT_ERROR_FROM_ERROR (self, err);
     return FALSE;
   }
-  g_mutex_unlock (&self->codec_lock);
   self->started = FALSE;
   self->flushing = TRUE;
 
@@ -702,7 +595,6 @@ gst_amc_video_enc_close (GstVideoEncoder * encoder)
 
   GST_DEBUG_OBJECT (self, "Closing encoder");
 
-  g_mutex_lock (&self->codec_lock);
   if (self->codec) {
     GError *err = NULL;
 
@@ -713,7 +605,6 @@ gst_amc_video_enc_close (GstVideoEncoder * encoder)
     gst_amc_codec_free (self->codec);
   }
   self->codec = NULL;
-  g_mutex_unlock (&self->codec_lock);
 
   self->started = FALSE;
   self->flushing = TRUE;
@@ -728,7 +619,6 @@ gst_amc_video_enc_finalize (GObject * object)
 {
   GstAmcVideoEnc *self = GST_AMC_VIDEO_ENC (object);
 
-  g_mutex_clear (&self->codec_lock);
   g_mutex_clear (&self->drain_lock);
   g_cond_clear (&self->drain_cond);
 
@@ -837,8 +727,7 @@ _find_nearest_frame (GstAmcVideoEnc * self, GstClockTime reference_timestamp)
       best_id = id;
 
       /* For frames without timestamp we simply take the first frame */
-      if ((reference_timestamp == 0 && !GST_CLOCK_TIME_IS_VALID (timestamp))
-          || diff == 0)
+      if ((reference_timestamp == 0 && timestamp == 0) || diff == 0)
         break;
     }
   }
@@ -888,7 +777,6 @@ gst_amc_video_enc_set_src_caps (GstAmcVideoEnc * self, GstAmcFormat * format)
 {
   GstCaps *caps;
   GstVideoCodecState *output_state;
-  GstStructure *s;
 
   caps = caps_from_amc_format (format);
   if (!caps) {
@@ -913,17 +801,6 @@ gst_amc_video_enc_set_src_caps (GstAmcVideoEnc * self, GstAmcFormat * format)
   if (!gst_video_encoder_negotiate (GST_VIDEO_ENCODER (self)))
     return FALSE;
 
-  output_state = gst_video_encoder_get_output_state (GST_VIDEO_ENCODER (self));
-  s = gst_caps_get_structure (output_state->caps, 0);
-
-  if (!strcmp (gst_structure_get_name (s), "video/x-h264") ||
-      !strcmp (gst_structure_get_name (s), "video/x-h265")) {
-    self->codec_data_in_bytestream = TRUE;
-  } else {
-    self->codec_data_in_bytestream = FALSE;
-  }
-  gst_video_codec_state_unref (output_state);
-
   return TRUE;
 }
 
@@ -959,8 +836,14 @@ gst_amc_video_enc_handle_output_frame (GstAmcVideoEnc * self,
    * gstomxvideoenc.c and gstomxh264enc.c */
   if ((buffer_info->flags & BUFFER_FLAG_CODEC_CONFIG)
       && buffer_info->size > 0) {
+    GstStructure *s;
+    GstVideoCodecState *state;
+
+    state = gst_video_encoder_get_output_state (encoder);
+    s = gst_caps_get_structure (state->caps, 0);
+    if (!strcmp (gst_structure_get_name (s), "video/x-h264")) {
+      gst_video_codec_state_unref (state);
 
-    if (self->codec_data_in_bytestream) {
       if (buffer_info->size > 4 &&
           GST_READ_UINT32_BE (buf->data + buffer_info->offset) == 0x00000001) {
         GList *l = NULL;
@@ -971,25 +854,20 @@ gst_amc_video_enc_handle_output_frame (GstAmcVideoEnc * self,
         hdrs = gst_buffer_new_and_alloc (buffer_info->size);
         gst_buffer_fill (hdrs, 0, buf->data + buffer_info->offset,
             buffer_info->size);
-        GST_BUFFER_PTS (hdrs) =
-            gst_util_uint64_scale (buffer_info->presentation_time_us,
-            GST_USECOND, 1);
 
         l = g_list_append (l, hdrs);
         gst_video_encoder_set_headers (encoder, l);
       }
     } else {
       GstBuffer *codec_data;
-      GstVideoCodecState *output_state =
-          gst_video_encoder_get_output_state (GST_VIDEO_ENCODER (self));
 
       GST_DEBUG_OBJECT (self, "Handling codec data");
 
       codec_data = gst_buffer_new_and_alloc (buffer_info->size);
       gst_buffer_fill (codec_data, 0, buf->data + buffer_info->offset,
           buffer_info->size);
-      output_state->codec_data = codec_data;
-      gst_video_codec_state_unref (output_state);
+      state->codec_data = codec_data;
+      gst_video_codec_state_unref (state);
 
       if (!gst_video_encoder_negotiate (encoder)) {
         gst_video_codec_frame_unref (frame);
@@ -1104,6 +982,15 @@ retry:
 
       gst_amc_format_free (format);
 
+      if (self->output_buffers)
+        gst_amc_codec_free_buffers (self->output_buffers,
+            self->n_output_buffers);
+      self->output_buffers =
+          gst_amc_codec_get_output_buffers (self->codec,
+          &self->n_output_buffers, &err);
+      if (!self->output_buffers)
+        goto get_output_buffers_error;
+
       if (idx >= 0)
         goto process_buffer;
 
@@ -1111,10 +998,18 @@ retry:
     }
 
     switch (idx) {
-      case INFO_OUTPUT_BUFFERS_CHANGED:
-        /* Handled internally */
-        g_assert_not_reached ();
+      case INFO_OUTPUT_BUFFERS_CHANGED:{
+        GST_DEBUG_OBJECT (self, "Output buffers have changed");
+        if (self->output_buffers)
+          gst_amc_codec_free_buffers (self->output_buffers,
+              self->n_output_buffers);
+        self->output_buffers =
+            gst_amc_codec_get_output_buffers (self->codec,
+            &self->n_output_buffers, &err);
+        if (!self->output_buffers)
+          goto get_output_buffers_error;
         break;
+      }
       case INFO_TRY_AGAIN_LATER:
         GST_DEBUG_OBJECT (self, "Dequeueing output buffer timed out");
         goto retry;
@@ -1137,30 +1032,24 @@ process_buffer:
       " flags 0x%08x", idx, buffer_info.size, buffer_info.presentation_time_us,
       buffer_info.flags);
 
-  buf = gst_amc_codec_get_output_buffer (self->codec, idx, &err);
-  if (err) {
-    if (self->flushing) {
-      g_clear_error (&err);
-      goto flushing;
-    }
-    goto failed_to_get_output_buffer;
-  } else if (!buf) {
-    goto got_null_output_buffer;
-  }
-
   frame =
       _find_nearest_frame (self,
       gst_util_uint64_scale (buffer_info.presentation_time_us, GST_USECOND, 1));
 
   is_eos = ! !(buffer_info.flags & BUFFER_FLAG_END_OF_STREAM);
 
+  if (idx >= self->n_output_buffers) {
+    GST_ERROR_OBJECT (self, "Invalid output buffer index %d of %d",
+        idx, self->n_output_buffers);
+
+    goto invalid_buffer;
+  }
+  buf = &self->output_buffers[idx];
+
   flow_ret =
       gst_amc_video_enc_handle_output_frame (self, buf, &buffer_info, frame);
 
-  gst_amc_buffer_free (buf);
-  buf = NULL;
-
-  if (!gst_amc_codec_release_output_buffer (self->codec, idx, FALSE, &err)) {
+  if (!gst_amc_codec_release_output_buffer (self->codec, idx, &err)) {
     if (self->flushing) {
       g_clear_error (&err);
       goto flushing;
@@ -1201,10 +1090,16 @@ dequeue_error:
     gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
     self->downstream_flow_ret = GST_FLOW_ERROR;
     GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
+    return;
+  }
+
+get_output_buffers_error:
+  {
+    GST_ELEMENT_ERROR_FROM_ERROR (self, err);
+    gst_pad_push_event (GST_VIDEO_ENCODER_SRC_PAD (self), gst_event_new_eos ());
+    gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
+    self->downstream_flow_ret = GST_FLOW_ERROR;
+    GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
     return;
   }
 
@@ -1219,10 +1114,6 @@ format_error:
     gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
     self->downstream_flow_ret = GST_FLOW_ERROR;
     GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
     return;
   }
 failed_release:
@@ -1232,10 +1123,6 @@ failed_release:
     gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
     self->downstream_flow_ret = GST_FLOW_ERROR;
     GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
     return;
   }
 flushing:
@@ -1255,45 +1142,25 @@ flow_error:
           gst_event_new_eos ());
       gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
     } else if (flow_ret == GST_FLOW_NOT_LINKED || flow_ret < GST_FLOW_EOS) {
-      GST_ELEMENT_FLOW_ERROR (self, flow_ret);
+      GST_ELEMENT_ERROR (self, STREAM, FAILED,
+          ("Internal data stream error."), ("stream stopped, reason %s",
+              gst_flow_get_name (flow_ret)));
       gst_pad_push_event (GST_VIDEO_ENCODER_SRC_PAD (self),
           gst_event_new_eos ());
       gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
     }
     GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
     return;
   }
 
-failed_to_get_output_buffer:
-  {
-    GST_ELEMENT_ERROR_FROM_ERROR (self, err);
-    gst_pad_push_event (GST_VIDEO_ENCODER_SRC_PAD (self), gst_event_new_eos ());
-    gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
-    self->downstream_flow_ret = GST_FLOW_ERROR;
-    GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
-    return;
-  }
-
-got_null_output_buffer:
+invalid_buffer:
   {
     GST_ELEMENT_ERROR (self, LIBRARY, SETTINGS, (NULL),
-        ("Got no output buffer"));
+        ("Invalid sized input buffer"));
     gst_pad_push_event (GST_VIDEO_ENCODER_SRC_PAD (self), gst_event_new_eos ());
     gst_pad_pause_task (GST_VIDEO_ENCODER_SRC_PAD (self));
-    self->downstream_flow_ret = GST_FLOW_ERROR;
+    self->downstream_flow_ret = GST_FLOW_NOT_NEGOTIATED;
     GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-    g_mutex_lock (&self->drain_lock);
-    self->draining = FALSE;
-    g_cond_broadcast (&self->drain_cond);
-    g_mutex_unlock (&self->drain_lock);
     return;
   }
 }
@@ -1305,7 +1172,7 @@ gst_amc_video_enc_start (GstVideoEncoder * encoder)
 
   self = GST_AMC_VIDEO_ENC (encoder);
   self->last_upstream_ts = 0;
-  self->drained = TRUE;
+  self->eos = FALSE;
   self->downstream_flow_ret = GST_FLOW_OK;
   self->started = FALSE;
   self->flushing = TRUE;
@@ -1330,11 +1197,17 @@ gst_amc_video_enc_stop (GstVideoEncoder * encoder)
     if (err)
       GST_ELEMENT_WARNING_FROM_ERROR (self, err);
     self->started = FALSE;
+    if (self->input_buffers)
+      gst_amc_codec_free_buffers (self->input_buffers, self->n_input_buffers);
+    self->input_buffers = NULL;
+    if (self->output_buffers)
+      gst_amc_codec_free_buffers (self->output_buffers, self->n_output_buffers);
+    self->output_buffers = NULL;
   }
   gst_pad_stop_task (GST_VIDEO_ENCODER_SRC_PAD (encoder));
 
   self->downstream_flow_ret = GST_FLOW_FLUSHING;
-  self->drained = TRUE;
+  self->eos = FALSE;
   g_mutex_lock (&self->drain_lock);
   self->draining = FALSE;
   g_cond_broadcast (&self->drain_cond);
@@ -1415,8 +1288,11 @@ gst_amc_video_enc_set_format (GstVideoEncoder * encoder,
   allowed_caps = gst_pad_get_allowed_caps (GST_VIDEO_ENCODER_SRC_PAD (encoder));
   if (!allowed_caps) {
     GST_DEBUG_OBJECT (self, "... but no peer, using template caps");
+    /* we need to copy because get_allowed_caps returns a ref, and
+     * get_pad_template_caps doesn't */
     allowed_caps =
-        gst_pad_get_pad_template_caps (GST_VIDEO_ENCODER_SRC_PAD (encoder));
+        gst_caps_copy (gst_pad_get_pad_template_caps (GST_VIDEO_ENCODER_SRC_PAD
+            (encoder)));
   }
   GST_DEBUG_OBJECT (self, "chose caps %" GST_PTR_FORMAT, allowed_caps);
   allowed_caps = gst_caps_truncate (allowed_caps);
@@ -1432,7 +1308,7 @@ gst_amc_video_enc_set_format (GstVideoEncoder * encoder,
       GST_STR_NULL (format_string));
   g_free (format_string);
 
-  if (!gst_amc_codec_configure (self->codec, format, NULL, &err)) {
+  if (!gst_amc_codec_configure (self->codec, format, 1, &err)) {
     GST_ERROR_OBJECT (self, "Failed to configure codec");
     GST_ELEMENT_ERROR_FROM_ERROR (self, err);
     goto quit;
@@ -1444,6 +1320,17 @@ gst_amc_video_enc_set_format (GstVideoEncoder * encoder,
     goto quit;
   }
 
+  if (self->input_buffers)
+    gst_amc_codec_free_buffers (self->input_buffers, self->n_input_buffers);
+  self->input_buffers =
+      gst_amc_codec_get_input_buffers (self->codec, &self->n_input_buffers,
+      &err);
+  if (!self->input_buffers) {
+    GST_ERROR_OBJECT (self, "Failed to get input buffers");
+    GST_ELEMENT_ERROR_FROM_ERROR (self, err);
+    goto quit;
+  }
+
   self->amc_format = format;
   format = NULL;
 
@@ -1461,7 +1348,7 @@ gst_amc_video_enc_set_format (GstVideoEncoder * encoder,
 
 quit:
   if (allowed_caps)
-    gst_caps_unref (allowed_caps);
+    gst_object_unref (allowed_caps);
 
   if (format)
     gst_amc_format_free (format);
@@ -1500,7 +1387,7 @@ gst_amc_video_enc_flush (GstVideoEncoder * encoder)
 
   /* Start the srcpad loop again */
   self->last_upstream_ts = 0;
-  self->drained = TRUE;
+  self->eos = FALSE;
   self->downstream_flow_ret = GST_FLOW_OK;
   gst_pad_start_task (GST_VIDEO_ENCODER_SRC_PAD (self),
       (GstTaskFunction) gst_amc_video_enc_loop, encoder, NULL);
@@ -1532,6 +1419,12 @@ gst_amc_video_enc_handle_frame (GstVideoEncoder * encoder,
     return GST_FLOW_NOT_NEGOTIATED;
   }
 
+  if (self->eos) {
+    GST_WARNING_OBJECT (self, "Got frame after EOS");
+    gst_video_codec_frame_unref (frame);
+    return GST_FLOW_EOS;
+  }
+
   if (self->flushing)
     goto flushing;
 
@@ -1541,16 +1434,6 @@ gst_amc_video_enc_handle_frame (GstVideoEncoder * encoder,
   timestamp = frame->pts;
   duration = frame->duration;
 
-  if (GST_VIDEO_CODEC_FRAME_IS_FORCE_KEYFRAME (frame)) {
-    if (gst_amc_codec_request_key_frame (self->codec, &err)) {
-      GST_DEBUG_OBJECT (self, "Passed keyframe request to MediaCodec");
-    }
-    if (err) {
-      GST_ELEMENT_WARNING_FROM_ERROR (self, err);
-      g_clear_error (&err);
-    }
-  }
-
 again:
   /* Make sure to release the base class stream lock, otherwise
    * _loop() can't call _finish_frame() and we might block forever
@@ -1583,6 +1466,9 @@ again:
     goto again;
   }
 
+  if (idx >= self->n_input_buffers)
+    goto invalid_buffer_index;
+
   if (self->flushing) {
     memset (&buffer_info, 0, sizeof (buffer_info));
     gst_amc_codec_queue_input_buffer (self->codec, idx, &buffer_info, NULL);
@@ -1602,17 +1488,11 @@ again:
 
   /* Copy the buffer content in chunks of size as requested
    * by the port */
-  buf = gst_amc_codec_get_input_buffer (self->codec, idx, &err);
-  if (err)
-    goto failed_to_get_input_buffer;
-  else if (!buf)
-    goto got_null_input_buffer;
+  buf = &self->input_buffers[idx];
 
   memset (&buffer_info, 0, sizeof (buffer_info));
   buffer_info.offset = 0;
   buffer_info.size = MIN (self->color_format_info.frame_size, buf->size);
-  gst_amc_buffer_set_position_and_limit (buf, NULL, buffer_info.offset,
-      buffer_info.size);
 
   if (!gst_amc_video_enc_fill_buffer (self, frame->input_buffer, buf,
           &buffer_info)) {
@@ -1621,14 +1501,9 @@ again:
     if (err && !self->flushing)
       GST_ELEMENT_WARNING_FROM_ERROR (self, err);
     g_clear_error (&err);
-    gst_amc_buffer_free (buf);
-    buf = NULL;
     goto buffer_fill_error;
   }
 
-  gst_amc_buffer_free (buf);
-  buf = NULL;
-
   if (timestamp != GST_CLOCK_TIME_NONE) {
     buffer_info.presentation_time_us =
         gst_util_uint64_scale (timestamp + timestamp_offset, 1, GST_USECOND);
@@ -1655,8 +1530,6 @@ again:
     goto queue_error;
   }
 
-  self->drained = FALSE;
-
   gst_video_codec_frame_unref (frame);
 
   return self->downstream_flow_ret;
@@ -1669,25 +1542,18 @@ downstream_error:
     gst_video_codec_frame_unref (frame);
     return self->downstream_flow_ret;
   }
-failed_to_get_input_buffer:
+invalid_buffer_index:
   {
-    GST_ELEMENT_ERROR_FROM_ERROR (self, err);
-    gst_video_codec_frame_unref (frame);
-    return GST_FLOW_ERROR;
-  }
-got_null_input_buffer:
-  {
-    GST_ELEMENT_ERROR (self, LIBRARY, SETTINGS, (NULL),
-        ("Got no input buffer"));
+    GST_ELEMENT_ERROR (self, LIBRARY, FAILED, (NULL),
+        ("Invalid input buffer index %d of %d", idx, self->n_input_buffers));
     gst_video_codec_frame_unref (frame);
     return GST_FLOW_ERROR;
   }
 buffer_fill_error:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, WRITE, (NULL),
-        ("Failed to write input into the amc buffer(write %dB to a %"
-            G_GSIZE_FORMAT "B buffer)", self->color_format_info.frame_size,
-            buf->size));
+        ("Failed to write input into the amc buffer(write %dB to a %dB buffer)",
+            self->color_format_info.frame_size, buf->size));
     gst_video_codec_frame_unref (frame);
     return GST_FLOW_ERROR;
   }
@@ -1715,10 +1581,57 @@ static GstFlowReturn
 gst_amc_video_enc_finish (GstVideoEncoder * encoder)
 {
   GstAmcVideoEnc *self;
+  gint idx;
+  GError *err = NULL;
 
   self = GST_AMC_VIDEO_ENC (encoder);
+  GST_DEBUG_OBJECT (self, "Sending EOS to the component");
 
-  return gst_amc_video_enc_drain (self);
+  /* Don't send EOS buffer twice, this doesn't work */
+  if (self->eos) {
+    GST_DEBUG_OBJECT (self, "Component is already EOS");
+    return GST_VIDEO_ENCODER_FLOW_DROPPED;
+  }
+  self->eos = TRUE;
+
+  /* Make sure to release the base class stream lock, otherwise
+   * _loop() can't call _finish_frame() and we might block forever
+   * because no input buffers are released */
+  GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
+  /* Send an EOS buffer to the component and let the base
+   * class drop the EOS event. We will send it later when
+   * the EOS buffer arrives on the output port.
+   * Wait at most 0.5s here. */
+  idx = gst_amc_codec_dequeue_input_buffer (self->codec, 500000, &err);
+  GST_VIDEO_ENCODER_STREAM_LOCK (self);
+
+  if (idx >= 0 && idx < self->n_input_buffers) {
+    GstAmcBufferInfo buffer_info;
+
+    memset (&buffer_info, 0, sizeof (buffer_info));
+    buffer_info.size = 0;
+    buffer_info.presentation_time_us =
+        gst_util_uint64_scale (self->last_upstream_ts, 1, GST_USECOND);
+    buffer_info.flags |= BUFFER_FLAG_END_OF_STREAM;
+
+    if (gst_amc_codec_queue_input_buffer (self->codec, idx, &buffer_info, &err)) {
+      GST_DEBUG_OBJECT (self, "Sent EOS to the codec");
+    } else {
+      GST_ERROR_OBJECT (self, "Failed to send EOS to the codec");
+      if (!self->flushing)
+        GST_ELEMENT_WARNING_FROM_ERROR (self, err);
+      g_clear_error (&err);
+    }
+  } else if (idx >= self->n_input_buffers) {
+    GST_ERROR_OBJECT (self, "Invalid input buffer index %d of %d",
+        idx, self->n_input_buffers);
+  } else {
+    GST_ERROR_OBJECT (self, "Failed to dequeue input buffer for EOS: %d", idx);
+    if (err)
+      GST_ELEMENT_WARNING_FROM_ERROR (self, err);
+  }
+
+  return GST_VIDEO_ENCODER_FLOW_DROPPED;
 }
 
 static GstFlowReturn
@@ -1734,9 +1647,9 @@ gst_amc_video_enc_drain (GstAmcVideoEnc * self)
     return GST_FLOW_OK;
   }
 
-  /* Don't send drain buffer twice, this doesn't work */
-  if (self->drained) {
-    GST_DEBUG_OBJECT (self, "Codec is drained already");
+  /* Don't send EOS buffer twice, this doesn't work */
+  if (self->eos) {
+    GST_DEBUG_OBJECT (self, "Codec is EOS already");
     return GST_FLOW_OK;
   }
 
@@ -1751,53 +1664,41 @@ gst_amc_video_enc_drain (GstAmcVideoEnc * self)
   idx = gst_amc_codec_dequeue_input_buffer (self->codec, 500000, &err);
   GST_VIDEO_ENCODER_STREAM_LOCK (self);
 
-  if (idx >= 0) {
-    GstAmcBuffer *buf;
+  if (idx >= 0 && idx < self->n_input_buffers) {
     GstAmcBufferInfo buffer_info;
 
-    buf = gst_amc_codec_get_input_buffer (self->codec, idx, &err);
-    if (buf) {
-      GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
-      g_mutex_lock (&self->drain_lock);
-      self->draining = TRUE;
+    GST_VIDEO_ENCODER_STREAM_UNLOCK (self);
+    g_mutex_lock (&self->drain_lock);
+    self->draining = TRUE;
 
-      memset (&buffer_info, 0, sizeof (buffer_info));
-      buffer_info.size = 0;
-      buffer_info.presentation_time_us =
-          gst_util_uint64_scale (self->last_upstream_ts, 1, GST_USECOND);
-      buffer_info.flags |= BUFFER_FLAG_END_OF_STREAM;
+    memset (&buffer_info, 0, sizeof (buffer_info));
+    buffer_info.size = 0;
+    buffer_info.presentation_time_us =
+        gst_util_uint64_scale (self->last_upstream_ts, 1, GST_USECOND);
+    buffer_info.flags |= BUFFER_FLAG_END_OF_STREAM;
 
-      gst_amc_buffer_set_position_and_limit (buf, NULL, 0, 0);
-      gst_amc_buffer_free (buf);
-      buf = NULL;
-
-      if (gst_amc_codec_queue_input_buffer (self->codec, idx, &buffer_info,
-              &err)) {
-        GST_DEBUG_OBJECT (self, "Waiting until codec is drained");
-        g_cond_wait (&self->drain_cond, &self->drain_lock);
-        GST_DEBUG_OBJECT (self, "Drained codec");
-        ret = GST_FLOW_OK;
-      } else {
-        GST_ERROR_OBJECT (self, "Failed to queue input buffer");
-        if (self->flushing) {
-          g_clear_error (&err);
-          ret = GST_FLOW_FLUSHING;
-        } else {
-          GST_ELEMENT_WARNING_FROM_ERROR (self, err);
-          ret = GST_FLOW_ERROR;
-        }
-      }
-
-      self->drained = TRUE;
-      self->draining = FALSE;
-      g_mutex_unlock (&self->drain_lock);
-      GST_VIDEO_ENCODER_STREAM_LOCK (self);
+    if (gst_amc_codec_queue_input_buffer (self->codec, idx, &buffer_info, &err)) {
+      GST_DEBUG_OBJECT (self, "Waiting until codec is drained");
+      g_cond_wait (&self->drain_cond, &self->drain_lock);
+      GST_DEBUG_OBJECT (self, "Drained codec");
+      ret = GST_FLOW_OK;
     } else {
-      GST_ERROR_OBJECT (self, "Failed to get buffer for EOS: %d", idx);
-      if (err)
+      GST_ERROR_OBJECT (self, "Failed to queue input buffer");
+      if (self->flushing) {
+        g_clear_error (&err);
+        ret = GST_FLOW_FLUSHING;
+      } else {
         GST_ELEMENT_WARNING_FROM_ERROR (self, err);
-      ret = GST_FLOW_ERROR;
+        ret = GST_FLOW_ERROR;
+      }
     }
+
+    g_mutex_unlock (&self->drain_lock);
+    GST_VIDEO_ENCODER_STREAM_LOCK (self);
+  } else if (idx >= self->n_input_buffers) {
+    GST_ERROR_OBJECT (self, "Invalid input buffer index %d of %d",
+        idx, self->n_input_buffers);
+    ret = GST_FLOW_ERROR;
   } else {
     GST_ERROR_OBJECT (self, "Failed to acquire buffer for EOS: %d", idx);
     if (err)
