@@ -99,6 +99,13 @@ static GstVTEncFrame *gst_vtenc_frame_new (GstBuffer * buf,
     GstVideoInfo * videoinfo);
 static void gst_vtenc_frame_free (GstVTEncFrame * frame);
 
+static void gst_pixel_buffer_release_cb (void *releaseRefCon,
+    const void *dataPtr, size_t dataSize, size_t numberOfPlanes,
+    const void *planeAddresses[]);
+
+static GstStaticCaps sink_caps =
+GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("{ NV12, I420 }"));
+
 static void
 gst_vtenc_base_init (GstVTEncClass * klass)
 {
@@ -124,14 +131,7 @@ gst_vtenc_base_init (GstVTEncClass * klass)
   g_free (description);
 
   sink_template = gst_pad_template_new ("sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      gst_caps_new_simple ("video/x-raw",
-          "format", G_TYPE_STRING, "NV12",
-          "width", GST_TYPE_INT_RANGE, min_width, max_width,
-          "height", GST_TYPE_INT_RANGE, min_height, max_height,
-          "framerate", GST_TYPE_FRACTION_RANGE,
-          min_fps_n, min_fps_d, max_fps_n, max_fps_d, NULL));
+      GST_PAD_SINK, GST_PAD_ALWAYS, gst_static_caps_get (&sink_caps));
   gst_element_class_add_pad_template (element_class, sink_template);
 
   src_caps = gst_caps_new_simple (codec_details->mimetype,
@@ -537,8 +537,6 @@ gst_vtenc_create_session (GstVTEnc * self)
 
   pb_attrs = CFDictionaryCreateMutable (NULL, 0, &kCFTypeDictionaryKeyCallBacks,
       &kCFTypeDictionaryValueCallBacks);
-  gst_vtutil_dict_set_i32 (pb_attrs, kCVPixelBufferPixelFormatTypeKey,
-      kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange);
   gst_vtutil_dict_set_i32 (pb_attrs, kCVPixelBufferWidthKey,
       self->negotiated_width);
   gst_vtutil_dict_set_i32 (pb_attrs, kCVPixelBufferHeightKey,
@@ -786,10 +784,11 @@ gst_vtenc_encode_frame (GstVTEnc * self, GstBuffer * buf)
 
     {
       const size_t num_planes = GST_VIDEO_FRAME_N_PLANES (&frame->videoframe);
-      void *plane_base_addresses[num_planes];
-      size_t plane_widths[num_planes];
-      size_t plane_heights[num_planes];
-      size_t plane_bytes_per_row[num_planes];
+      void *plane_base_addresses[GST_VIDEO_MAX_PLANES];
+      size_t plane_widths[GST_VIDEO_MAX_PLANES];
+      size_t plane_heights[GST_VIDEO_MAX_PLANES];
+      size_t plane_bytes_per_row[GST_VIDEO_MAX_PLANES];
+      OSType pixel_format_type;
       size_t i;
 
       for (i = 0; i < num_planes; i++) {
@@ -803,18 +802,27 @@ gst_vtenc_encode_frame (GstVTEnc * self, GstBuffer * buf)
             GST_VIDEO_FRAME_COMP_STRIDE (&frame->videoframe, i);
       }
 
+      switch (GST_VIDEO_INFO_FORMAT (&self->video_info)) {
+        case GST_VIDEO_FORMAT_I420:
+          pixel_format_type = kCVPixelFormatType_420YpCbCr8Planar;
+          break;
+        case GST_VIDEO_FORMAT_NV12:
+          pixel_format_type = kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange;
+          break;
+        default:
+          goto cv_error;
+      }
+
       cv_ret = CVPixelBufferCreateWithPlanarBytes (NULL,
           self->negotiated_width, self->negotiated_height,
-          kCVPixelFormatType_420YpCbCr8BiPlanarVideoRange,
-          NULL,
+          pixel_format_type,
+          frame,
           GST_VIDEO_FRAME_SIZE (&frame->videoframe),
           num_planes,
           plane_base_addresses,
           plane_widths,
           plane_heights,
-          plane_bytes_per_row,
-          (CVPixelBufferReleaseBytesCallback) gst_vtenc_frame_free, frame,
-          NULL, &pbuf);
+          plane_bytes_per_row, gst_pixel_buffer_release_cb, frame, NULL, &pbuf);
       if (cv_ret != kCVReturnSuccess) {
         gst_vtenc_frame_free (frame);
         goto cv_error;
@@ -897,7 +905,9 @@ gst_vtenc_enqueue_buffer (void *data, int a2, int a3, int a4,
   }
   self->expect_keyframe = FALSE;
 
-  buf = gst_core_media_buffer_new (sbuf);
+  /* We are dealing with block buffers here, so we don't need
+   * to enable the use of the video meta API on the core media buffer */
+  buf = gst_core_media_buffer_new (sbuf, FALSE);
   gst_buffer_copy_into (buf, self->cur_inbuf, GST_BUFFER_COPY_TIMESTAMPS,
       0, -1);
   if (is_keyframe) {
@@ -957,6 +967,15 @@ gst_vtenc_frame_free (GstVTEncFrame * frame)
   g_slice_free (GstVTEncFrame, frame);
 }
 
+static void
+gst_pixel_buffer_release_cb (void *releaseRefCon, const void *dataPtr,
+    size_t dataSize, size_t numberOfPlanes, const void *planeAddresses[])
+{
+  GstVTEncFrame *frame = (GstVTEncFrame *) releaseRefCon;
+
+  gst_vtenc_frame_free (frame);
+}
+
 static void
 gst_vtenc_register (GstPlugin * plugin,
     const GstVTEncoderDetails * codec_details)
